from visual import *
import time



def setupSpheres(count=50):
    global ptsary,sphrs
    ptsary = []
    for x in range(count):
        for y in range(count):
            for z in range(count):
                ptsary.append((x,y,z))
    sphrs = [sphere(pos=x,color=vector(x)/count, size=1) for x in ptsary]


def setupPoints(count=50):
    global ptsary,pts
    ptsary = []
    colorary = []
    for x in range(count):
        for y in range(count):
            for z in range(count):
                ptsary.append((x,y,z))
                colorary.append((1.0*x/count,1.0*y/count,1.0*z/count))
    pts = points(pos=ptsary,color=colorary,size=10, size_units="world")

def testColorChangeSpheres(color):
    global sphrs
    start = time.time()
    for sphr in sphrs:
        sphr.color = color
    stop = time.time()
    return stop-start


def testColorChangePoints(color):
    global pts
    start = time.time()
    colors = pts.get_color()
    for cidx in xrange(len(colors)):
        colors[cidx] = color
    stop = time.time()
    return stop-start

#scene.stereodepth=2
#scene.stereo="redcyan"



#!/usr/bin/ipython

import sys
import time
import vtrace
import vtrace.notifiers as vt_n
import atlasutils.vtraceutils as av

pattern = sys.argv[1]


s = vtrace.getTrace()
current = []

def calibrate():
	global current
	for pid,name in s.ps():
		if pattern in name and pid not in current:
			current.append(pid)

calibrate()

pid = None

def rotate(stopOnExc=False):
	while True:
	    wait()
            if go(stopOnExc): return

def go(stopOnExc=False):
    try:
	while True:
	    t.run()
	    if not t.isAttached(): break

            try:
	        print av.printStuff(t)
            except Exception, e:
                print e
	    
            #fixme: if exception in tracer, drop to interactive mode
            if t.metadata['PendingSignal'] != None and stopOnExc:
                print "Caught Signal, returning to interactive python (you did use -i or ipython, right?."
                return 1

    except KeyboardInterrupt:
	print "Caught Ctrl-C, returning to interactive python (if you ran it that way)."
        t.sendBreak()
        t.running = False
	return 1
    except:
	sys.excepthook(*sys.exc_info())
    return 0


def wait():
	global current, t, s, pattern, pid

	t = vtrace.getTrace()
	print("Current PIDs")
	print(repr(current))

	pid = None

	print "waiting for new child..."
	while pid==None:
		#print(repr(current))
		for ppid,name in s.ps():
			if ppid not in current and pattern in name:
				pid = ppid
				break

	time.sleep(.3)
	t.attach(pid)
        t.registerNotifier(vtrace.NOTIFY_ALL, vt_n.VerboseNotifier())
	t.sendBreak()
	print("Caught one.  Attaching to %d" % pid)


print "wait() or rotate()"




from sys import *
from vtrace import *
from select import select
from vtrace.breakpoints import *
from atlasutils.smartprint import *
from atlasutils import *
try:
    from atlasutils.disassutils import *
    from disassemble import *
except:
    print(exc_info())
    print("Initializing vtraceutils *without* libdisassemble... this could be bad.")

BITS = 32
AddrFmt='L'
AddrBytes = BITS/8
AddrHeapMallocMask = -8
EXAMPLE_MEMORY_LENGTH = 24

def setBits(bits):
    BITS=bits
    AddrFmt=('B','H',None,'L',None,None,None,'Q')[BITS/8]
    AddrBytes = BITS/8
    AddrHeapMallocMask = -8
    EXAMPLE_MEMORY_LENGTH = 24


class DisplayBreaker(Breakpoint):
    def __init__(self, address):
        Breakpoint.__init__(self, address)
        self.address = address
        
    def notify(self, event, trace):
        trace.running = False
        print("PID: %d hit break: %s" % (trace.pid, hex(self.address)))
        select([],[],[],1)
        print(printStuff(trace))
        print(XI(trace, self.address, 5))
        
class StackSysCallBreaker(Breakpoint):
    def __init__(self, tracer, syscallname = 'libc.memcpy'):
        address = tracer.parseExpression(syscallname)
        Breakpoint.__init__(self, address)
    def notify(self, event, trace):
        esp = trace.getRegisterByName('esp')
        SmartPrint(trace.readMemory(esp, 200))

##### SET BREAKPOINTS FOR ALL "cmp %eax, *"
def breakOnAll(asmdict, opstring = "cmp %eax", trace = None):
    """
    searches through the assembly code (in the form of the dict asmdict) and adds vtrace.Breakpoints for each occurrence of the "opstring"
    asmdict:   keys = "<hexaddress>",   values = "<opcode string>"
        eg.   asmdict.setdefault("0x401443", "cmp %eax, $0x12345")
    
    returns an array of keys where the values match.
    """
    cmps = []
    keys = asmdict.keys()
    values = asmdict.values()
    for i in range(len(values)):
        if (values[i][:len(opstring)] == opstring):
            print(" Adding breakpoint for  %s: %s"%(keys[i],values[i])) 
            cmps.append(keys[i])
            if trace: trace.addBreakpoint(Breakpoint(int(keys[i],16)))
    return (cmps)


def brk(me):
    me.running=True
    for t in me.getThreads():
        me.sendBreak()
    me.running=False
    

def stepi(me):
    origThread = me.getMeta('ThreadId')
    for t in me.getThreads():
        me.selectThread(t)
        try:
            me.stepi()
            print("thread %8d: %x"%(t,me.getProgramCounter()))
        except Exception as e:
            print("thread %8d: ERROR: %s"%(t, e))
    me.selectThread(origThread)

def prettystepi(me, outfile = stderr):
    origThread = me.getMeta('ThreadId')
    for t in me.getThreads():
        me.selectThread(t)
        me.stepi()
        eip = me.getProgramCounter()
        print("thread %8d: %x   (%s)"%(t,eip,me.getSymByAddr(eip)), file=outfile)
    me.selectThread(origThread)

def eips(me):
    origThread = me.getMeta('ThreadId')
    for t in me.getThreads():
        me.selectThread(t)
        print("thread %8d: %x"%(t,me.getProgramCounter()))
    me.selectThread(origThread)









LOT_STRINGOUTPUT = "s"
LOT_INSTRUCTIONOUTPUT = "i"
LOT_HEXOUTPUT = "x"


def liveOrganTransplant(pidortrace, memloc, count = None, style = 'i'):
    output = ""
    if type(memloc) == str:  
        if memloc.lower().find('x') > -1:
            memloc = int(memloc,16)
        else:
            memloc = int(memloc)
    if type(count) == str:
        if count.lower().find('x') > -1:
            count = int(count,16)
        else:
            count = int(count)

    if (type(pidortrace) == int):
        trace = atlasutils.vtraceutils.getTrace()
        trace.attach(pidortrace)
    else:
        trace = pidortrace
    if style == LOT_STRINGOUTPUT:
        output = atlasutils.vtraceutils.XS(trace, memloc, count)
    elif style == LOT_HEXOUTPUT:
        output = atlasutils.vtraceutils.XW(trace, memloc, count)
    else:
        output = atlasutils.vtraceutils.XI(trace, memloc, count)
    if (type(pidortrace) == int):
        trace.detach()
    return output




locs = {}
def traceme(me, untilop=None, untileip=None, untilreg=('eax',None), locs = {}, printTrace=None, printAllThreads=False, triggerOnAnyThread = True, out=stdout, err=stderr, disasstrace=False):
    """
    So many options, let's see if we can explain some of them.
        me                  - Vtrace object to trace
            If specified, the following 'until-" options will cause the trace to terminate.  This can be for any thread, or just the original thread, depending on the value of 'triggerOnAnyThread'
        untilop             - a string that is checked against the current opcode for all threads (or just one)
        untileip            - a number checked against the program counter
        untilreg            - a tuple ('eax', value) where a register is value-checked
        
        locs                - keeps track of jmps/calls and targets, as well as the hit-count.  Hand in if you wish, or use the default of {}
        printTrace          - prints out a pretty list of 'locs' in order
        printAllThreads     - cycle through all threads with "printStuff(me)"
        triggerOnAnyThread  - Apply 'until-' clauses to any thread.  If False, only the original thread will be compared to stop tracing.
    """
    if (not (printTrace or untilop or untileip or untilreg[1])):
        printTrace=True #set the default.  If none specified, print(if no until-s.  If there are until-s, default to no printing unless specified)
        
    print("TRACING EXECUTION... Press CTRL-C to stop.  Accounting will be stored in 'locs' and returned", file=err)
    origThread = me.getMeta('ThreadId')
    last=me.getProgramCounter()
    cont=True
    eip=None
    op=None
    disasstracer = None
    for x in me.getThreads():
        locs[x] = {}
    if disasstrace:
        disasstracer = {}
    try:
        while cont:
            t = me.getThreads()
            try:
                for i in t:
                    me.selectThread(i)
                    me.stepi()
                    if (printAllThreads or i == origThread):
                        print(printStuff(me), file=out)
                        print( t, file=out)
                        
                        
                    if (triggerOnAnyThread or i == origThread):
                        eip=me.getProgramCounter()
                        op=Opcode(me.readMemory(eip, 14))
                        if disasstrace:
                            ### Let's save some state information.  This gets big fast
                            cur = disasstracer.get(eip, [op.printOpcode(0)])
                            if op.source:
                                recurseOperand(me, op.source, cur, i)
                                    
                            if op.dest:
                                recurseOperand(me, op.dest, cur, i)
                                    
                            if op.aux:
                                recurseOperand(me, op.aux, cur, i)
                                    
                            disasstracer[eip] = cur
                        if (abs(eip - last) > 9):
                            locs[i][last] = (locs[i].get(last,0)) + 1
                            locs[i][eip] = (locs[i].get(eip,0)) + 1
                        last=eip
                        
                        if (untilop):
                                #if (untilop in getinst(me)):
                                if (op.opcode in untilop):
                                    cont=False
                                    break
                        if (untileip):
                                if (untileip == me.getProgramCounter()):
                                    cont=False
                                    break
                        if (untilreg[1]):
                                if (me.getRegisters()[untilreg[0]] == untilreg[1]):
                                    cont=False
                                    break
            except KeyboardInterrupt as  k:
                me.selectThread(origThread)
                break
            except:
                print("**************======== Opcode Not Printed... (bad or not supported?) ========**************", file=stderr)
                x,y,z = exc_info()
                excepthook(x,y,z)
            
            
            
    except KeyboardInterrupt as  e:
        print("Trace Complete.  Look at 'locs'", file=err)
    if printTrace:
        printLocs(locs, out)
    if disasstrace:
        return disasstracer
    return locs

def recurseOperand(me, oper, cur, i):
    if (isinstance(oper, Expression)):
        #  Expressions have:  base.index, base.base
        if oper.disp:
            recurseOperand(me, oper.disp, cur, i)
        if oper.base:
            recurseOperand(me, oper.base, cur, i)
    if (isinstance(oper, Register)):
        #  Registers have 'name' which should fit into vtrace nicely
        cur.append("(thd%d)%s=%x"%(i, oper.name, me.getRegisterByName(oper.name)))
    elif (isinstance(oper, SIB)):
        if oper.base:
            recurseOperand(me, oper.base, cur, i)
        if oper.index:
            recurseOperand(me, oper.index, cur, i)
            
def getOperandValue(trace, opcode, operand, eip):
        dereference = False
        add = ""
        numaddr = 0
        source = operand
        if isinstance(source, SIB):
            if source.index:
                numaddr += (souce.index * source.scale)
            if source.base:
                source = source.base
            #print("DEBUG-sib: %x"%numaddr, file=outfile)
        if isinstance(source, Expression):
            if source.disp:
                numaddr += source.disp.value
            if source.base:
                source = source.base
            dereference = True
            #print("DEBUG-expr: %x"%numaddr, file=outfile)
        if isinstance(source, SIB):
            if source.index:
                numaddr += (souce.index * source.scale)
            if source.base:
                source = source.base
            #print("DEBUG-sib: %x"%numaddr, file=outfile)
        if isinstance(source, Register):
            reg = source.name
            mask = 0xffffffff       # 32 bit only
            if reg[0] != 'e' and reg[1] in ['l','h','x']:       # TOTALLY libdisassemble-centric, as it assumes lowercase letters
                if reg[1] == 'x':
                    mask = 0xffff
                elif reg[1] == 'l':
                    mask = 0xff
                elif reg[1] == 'h':
                    mask = 0xff00
                reg = 'e%cx'%reg[0]                             # vtrace doesn't like al/ah and possibly ax
            numaddr += (trace.getRegisterByName(reg) & mask)
            #print("DEBUG-reg: %x"%numaddr, file=outfile)
        elif isinstance(source, Address):
            numaddr += source.value
            if source.relative:
                numaddr += eip + opcode.off
            #print("DEBUG-addr: %x"%numaddr, file=outfile)
        if dereference:
            try:
                numaddr = struct.unpack("L",me.readMemory(numaddr, 4))[0]
                #print("DEBUG-deref: %x"%numaddr, file=outfile)
            except:
                #print("DEBUG-deref-except: %x"%numaddr, file=outfile)
                #print("DEBUG-deref-except: %x"%numaddr, file=stderr)
                pass
                
        return numaddr
    


def printLocs(locs, out=stdout):
    keys = locs.keys()
    keys.sort()
    for t in locs.keys(): 
        lkeys = locs[t].keys()
        lkeys.sort()
        for i in lkeys:
            print("(Thread %d): %x:%d"%(t, i, locs[t][i]), file=out)


def gotoOpcode(me, inbtwn=False, srch=['call'], file = stdout):
    """
    stepi() this thread until the opcode is found in srch
    This is single-threaded only at this point.  Only the current thread will be incremented. 
    """
    output = ""
    GO = True
    PRINT = True
    while (GO):
      for t in me.getThreads():
        me.selectThread(t)
        me.stepi()
        p = Opcode(me.readMemory(me.getProgramCounter(),14))
        if (inbtwn or PRINT): 
            file.write("\neip: %.08x: %s\n"%(me.getProgramCounter(),getinst(me)))
            PRINT=False
        for i in srch:
            if (p.opcode == i): GO = False

def walkTheLine(me, VERBOSE = False, file = stdout):
    while (True):
        gotoOpcode(me, VERBOSE, ['call'], file)
        p = Opcode(me.readMemory(me.getProgramCounter(),14))
        file.write(printStuff(me))
        #file.write(("%x: %s"%(me.getProgramCounter(),p.printOpcode("ATT")))

def getinst(me):
    eip=me.getProgramCounter()
    p = Opcode(me.readMemory(eip,14))
    return p.printOpcode("ATT", eip)


def si(me):
    me.stepi()
    print(printStuff(me))


def ni(me):
    eip = me.getProgramCounter()
    #print("debug: %x"%self.eip, file=outfile)
    op = Opcode(me.readMemory(eip, 18))
    if op.opcode[:2] == "ca":
        baddr = eip+op.off
        brk = OneTimeBreak(baddr)
        #print("\t\t Going Remote...  Setting Breakpoint at %x"%(baddr), file=outfile)
        me.addBreakpoint(brk)
        me.run()
        #print(repr(me))
        #me.removeBreakpoint(brk)
    else:
        me.stepi()
    print(printStuff(me))

            

def XW(tracer, address, length = 32, dwperline = 8):
    output = ""
    for i in range(length):
        if (i % dwperline == 0):
            output += "%.08x:\t "%(address+(i*4))
        bs = tracer.readMemory(address+(i*4),4)
        for x in range(3, -1,-1):
            output += "%.02x"%(ord(bs[x]))
        if ((i+1) % dwperline == 0):
            output += "\n"
        else:
            output += "  "
    return output
        


def XI(tracer, address, length = 20, mode=32):
    output = ""
    offset = 0
    last = None
    prev = None
    p = {}
    for i in range(length):
        buffer = buffer = tracer.readMemory(address+offset,14)
        prev = last
        last = p
        try:
            p = Opcode(buffer)
        except:
            p.opcode = "ERROR: %s"%exc_info()[0]
            p.off = 1
            
        bytes = ""
        for i in buffer[:p.getSize()]:
            bytes += "%.02x "%ord(i)
        
        output += "%x: %30s\t %s\n"%((address+offset),bytes,p.printOpcode("AT&T",address+offset))
        if not p:
            break
        offset += p.getSize()
    return output

def XS(tracer, address, length=1, BUFFLEN=16):
    output = "%x: "%address
    offset = 0
    for i in range(length):
        eol = False
        while not eol:
            buffer = tracer.readMemory(address+offset,BUFFLEN)
            where = buffer.find("\x00")
            if (where>-1):
                offset += 1+where
                output += "%s\n%x: "%(buffer[:where], address+offset)
                eol = True
            else:
                offset += BUFFLEN
                output += buffer
    return output

REG_EAX = 0
REG_ECX = 1
REG_EDX = 2
REG_EBX = 3
REG_ESI = 4
REG_EDI = 5
REG_EIP = 6
REG_ESP = 7
REG_EBP = 8    
def printStuff(me, eachThread=False, width=8):
    output = ""
    iterate = [me.metadata['ThreadId']]
    if eachThread:
        iterate = me.getThreads()
    
    for t in iterate:
        frame = {'ebp': me.getRegisterByName("ebp"), 'esp': me.getRegisterByName("esp"),
                 'eax': me.getRegisterByName("eax"), 'ebx': me.getRegisterByName("ebx"),
                 'ecx': me.getRegisterByName("ecx"), 'edx': me.getRegisterByName("edx"),
                 'esi': me.getRegisterByName("esi"), 'edi': me.getRegisterByName("edi"),
                 'eip': me.getProgramCounter() }
        
        output += "\nThreadId:\t %d"%(t)
        output += "\n%esp: " + "(%x)\n" % frame.get('esp')
        try:
            output += XW(me, frame.get('esp'), 32, width)
        except:
            output += "ERROR READING FROM MEMORY AT %x\n"%frame.get('esp')
        
        output += "\n%ebp-92: " + "(%x)\n" % frame.get('esp')
        try:
            output += XW(me, frame.get('ebp')-92, 32, width)
        except:
            output += "ERROR READING FROM MEMORY AT %x\n"%(frame.get('ebp')-92)
        
        for i in ['eax','ecx','edx','ebx','esi','edi']:
            output += "%s: %.08x\t"%(i,frame.get(i))
        
        cmd = me.getSymByAddr(frame['eip'])
        output += "\neip: %.08x: %s\t\t\t%s"%(frame.get('eip'), getinst(me), cmd)
        output += "\n\t(%8x (%x): %d  %s)"%me.getMemoryMap(frame['eip'])
    return output


def atch(procname, me = getTrace(), procno = None):
  for proc in me.ps() :
    if (proc[1].split(" ")[0].find(procname) >-1):
      print(("Attaching to %d: %s"%(proc[0],proc[1])))
      try:
        me.attach(proc[0])  
        me.registerNotifier(vtrace.NOTIFY_ALL, VerboseNotifier())
        print("isRunning: %s\t\tisAttached: %s"%(me.isRunning(),me.isAttached()))
        print("Threads: %s"%(str(me.getThreads())))
        bt(me)
        return me
      except:
        print("Error attaching to pid:")
        print(proc)
        excepthook(exc_info()[0], exc_info()[1],exc_info()[2])
  return None

def bt(me):
    print("Stack Trace: (current thread)")
    for i in me.getStackTrace(): 
        print("%x : %x"%i)
        

def findRET(trace, stackptr = 0):
    """
    findRET() traces up the stack searching for the previous RET address.  By default, findRET assumes that this is called from a breakpoint immediately at a system call before the stack is modified (ie, we assume there is a RET at %ESP so we skip ahead four bytes)
    If you hand in the stack pointer, this will find the RET value preceding that.
    """
    try:
        cont = True
        if stackptr == 0:
            stackptr = trace.getRegisterByName('esp')
        orig = stackptr
        stderr.write("Call to findRET (0x%x) "%stackptr)
        while cont:
            stackptr += 1#4
            if stackptr - orig > 1000:
                trace.setMode("RunForever", False)
            address = trace.readMemoryFormat(stackptr, AddrFmt)[0]
            mmap = trace.getMemoryMap(address)
            # valid address in an executable memory map?
            if mmap != None and mmap[2]&1:   
                #bytes just before the RET address(the opcode before, should be call)
                buf = buffer(trace.readMemory(address-8, 24))
                BUFLEN = 8
                for x in xrange(BUFLEN,0,-1):
                    opchar = buf[x]
                    #stderr.write("  %x%x  "%(x,ord(opchar)))
                    if (opchar == '\xff' and (ord(buf[x+1])>>4) & 3 == 1) or opchar == '\xe8' or opchar == '\x9a':
                        #print(hexText(buf[x:]), file=stderr)
                        op = Opcode(buf, x)
                        #stderr.write("*(%x:%x:%s:%d+%d=%d)\n"%(stackptr,address, op.opcode, op.getSize(), x,BUFLEN+1))
                        #if the length is correct and it's a call....  take our win... in one of two ways
                        if (op.opcodetype == opcode86.INS_CALL and op.getSize() + x == BUFLEN): 
                            if (op.source.opertype == 1 or op.source.opertype == 3):  #OPER_REGISTER or OPER_EXPRESSION
                                return stackptr
                            else:
                                #  recurses the Operand information to return a real value
                                target = getOperandValue(trace, op, op.source, address-op.off)  
                                #print("\n"+op.printOpcode(0)+"\n%x : %x"%(address, target), file=stderr)
                                mmap = trace.getMemoryMap(target)
                                if mmap != None and "stack" not in mmap[3] :        #  Is it a valid address?
                                    # Possibly Check the Target of the call
                                    #   * Does it point to this address or a opcode for a jmp to this address?
                                    #       Unfortunately, this is costly and not entirely accurate
                                    print("foundRET: (s/r/c)\t%x:%x:%x (%s - %s)\n"%(stackptr,address,target, op.printOpcode(0, address-10+x), trace.getSymByAddr(target)), file=stderr)
                                    return stackptr
                        else:
                            pass
                            #stderr.write("--%d--%d--"%(op.opcodetype,opcode86.INS_CALL))
            else:
                stderr.write(".")
    except:
        errorhandler()

"""
trace=me
esp = trace.getRegisterByName('esp')
op=None
#### loop through RET and see what goes where, and print(the opcode preceeding ret)
try:
    while True:
        address = findRET(me,esp)
        buf = trace.readMemory(address-10, 20)      #bytes just before the RET address(the opcode before, should be call)
        esp = address
except:
    pass
"""
import sys

ERROR_COUNT=0

def errorhandler():
    global ERROR_COUNT
    ERROR_COUNT += 1
    x,y,z = sys.exc_info()
    sys.excepthook(x,y,z)
    



import envi
import envi.archs.i386 as e_i386
import envi.archs.i386.opcode86 as i3op
import envi.archs.amd64 as e_amd64
import envi.archs.amd64.opcode86 as i64op



def asm(instr):
    pieces = instr.split()
    nmem = pieces.pop(0)

    prefix = []
    while mnem in (PREFIXES):
        prefix.append(mnem)
        nmem = pieces.pop(0)

    raise (Exception("Not completed."))



#server
import socket, select
s=socket.socket()
#s.setsockopt(socket.SOL_SOCKET,SO_REUSEADDR,1)
s.bind(('localhost',1366))
s.listen(100)
si,addy=s.accept()
si.setsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR,1)
si.send("This is a test server.  Totally oblivious to any other protocols, I'm a dummy.")




import logging
import vivisect
logging.basicConfig(level=logging.INFO)


def immerge(vw0, vw1, subs=None):
    if subs is None:
        subs = {}

    for filenm in vw1.getFiles():
        vw0.vprint("merging VW file: %r" % filenm)
        # do substitutions
        tgtfilenm = subs.get(filenm, filenm)

        if tgtfilenm in vw0.getFiles():
            vw0.vprint("    found!")
            vw0ib = vw0.getFileMeta(tgtfilenm, 'imagebase')
            vw1ib = vw1.getFileMeta(filenm, 'imagebase')    # what about multiples?  oh, file should be based correctly, we hope
            imgdelta = vw0ib - vw1ib
            vw0.vprint("(va delta: 0x%x)" % imgdelta)

            # do names first
            vw0.vprint("Merging Names...")
            for va1, name in vw1.name_by_va.items():
                rebaseva = va1 + imgdelta
                vw0.vprint("0x%x: %r   0x%x: %r" % (va1, name, rebaseva, vw0.getName(rebaseva)))
                if name.endswith("_%.8x" % va1):
                    name = name[:-9] + "_%.8x" % rebaseva
                vw0.vprint("=> 0x%x:  %r" % (rebaseva, name))

                curname = vw0.getName(rebaseva)
                if curname == name:
                    continue

                if curname is not None and curname != name and curname != "sub_%.8x" % rebaseva:
                    whichname = None
                    while whichname not in ('1', '2'):
                        whichname = input("0x%x:  (1) %r    (2) %r    choose: " % (rebaseva, curname, name))
                        vw0.vprint("your answer: %r" % whichname)

                    if whichname == '1':
                        continue

                vw0.makeName(rebaseva, name)

            # now roll through locations:
            #'LOC_STRUCT',
            # import any structure namespaces
            ct_struct = 0
            for lva, lsz, lt, lti in vw1.getLocations(vivisect.LOC_STRUCT):
                print("LOC_STRUCT: %r  %r", hex(lva), lti)
                ct_struct += 1

            #'LOC_PAD',
            # nahhhh, skip it.

            #'LOC_IMPORT',
            #  we may not want imports, if we've imported the whole memoryspace

            #'LOC_OP',
            # skip these... assume they're identified in makeFunction

            print('LOC_NUMBER')
            ct_num = 0
            for lva, lsz, lt, lti in vw1.getLocations(vivisect.LOC_NUMBER):
                #print("LOC_NUMBER", hex(lva), lsz, lti)
                tloc = vw0.getLocation(lva+imgdelta)
                if tloc is not None:
                    tlva, tlsz, tlt, tlti = tloc
                    if tlt != lt or tlti != lti:
                        vw0.vprint("number failure: 0x%x: %r %r" % (lva, repr((lva, lsz, lt, lti)), repr(tloc)))
                        continue
                vw0.makeNumber(lva + imgdelta, lsz)
                ct_num += 1

            print('LOC_POINTER')
            ct_ptr = 0
            for lva, lsz, lt, lti in vw1.getLocations(vivisect.LOC_POINTER):
                #print("LOC_POINTER", hex(lva), lti)
                if lti is not None:
                    lti += imgdelta
                tloc = vw0.getLocation(lva+imgdelta)
                if tloc is not None:
                    tlva, tlsz, tlt, tlti = tloc
                    if tlt != lt or tlti != lti:
                        input("pointer failure: 0x%x: %r %r" % (lva, repr((lva, lsz, lt, lti)), repr(tloc)))
                        continue

                if lti:
                    vw0.makePointer(lva + imgdelta, tova=lti)
                else:
                    vw0.makePointer(lva + imgdelta)
                ct_ptr += 1

            print('LOC_UNI')
            ct_uni = 0
            for lva, lsz, lt, lti in vw1.getLocations(vivisect.LOC_UNI):
                #print("LOC_UNI", hex(lva), lti)
                tloc = vw0.getLocation(lva+imgdelta)
                if tloc is not None:
                    tlva, tlsz, tlt, tlti = tloc
                    if tlt != lt or tlti != lti:
                        vw0.vprint("uni failure: 0x%x: %r %r" % (lva, repr((lva, lsz, lt, lti)), repr(tloc)))
                        continue
                vw0.makeUnicode(lva + imgdelta)
                ct_uni += 1

            print('LOC_STRING')
            ct_str = 0
            for lva, lsz, lt, lti in vw1.getLocations(vivisect.LOC_STRING):
                #print("LOC_STRING", hex(lva), lti)
                tloc = vw0.getLocation(lva+imgdelta)
                if tloc is not None:
                    tlva, tlsz, tlt, tlti = tloc
                    if tlt != lt or tlti != lti:
                        vw0.vprint("string failure: 0x%x: %r %r" % (lva, repr((lva, lsz, lt, lti)), repr(tloc)))
                        continue
                vw0.makeString(lva + imgdelta)
                ct_str += 1


            # make functions
            print('FUNCTIONS')
            ct_funcs = 0
            for fva in vw1.getFunctions():
                newfva = fva + imgdelta
                # do a few checks first
                loc = vw1.getLocation(fva)
                if loc is None:
                    continue
                lva, lsz, lt, lti = loc
                if lt != vivisect.LOC_OP:
                    continue
                # finally, grab the arch (useful for ARMv7)
                arch = lti
                vw0.makeFunction(newfva, arch=arch)
                ct_funcs += 1

            # should we do XREFS that aren't there yet???
            print('XREFS')
            ct_xrefs = 0
            for xref in vw1.getXrefs():
                xfr, xto, xt, xtflag = xref
                xfr += imgdelta
                xto += imgdelta
                skip = False
                for xr2 in vw0.getXrefsFrom(xfr):
                    if xr2 == (xfr, xto, xt, xtflag):
                        skip = True

                if skip:
                    continue

                vw0.addXref(xfr, xto, xt, xtflag)
                ct_xrefs += 1

            # VaSets?
            print('VaSets')
            for vaset in vw1.getVaSetNames():
                if vaset not in vw0.getVaSetNames():
                    defs = vw1.getVaSetDef(vaset)
                    vw0.addVaSet(vaset, defs)
                print("... %r" % vaset)
                for vasetrow in vw1.getVaSetRows(vaset):
                    newrow = (vasetrow[0] + imgdelta, ) + vasetrow[1:]
                    vw0.setVaSetRow(vaset, newrow)


            vw0.setFileMeta(tgtfilenm, 'merged', True)
        else:
            vw0.vprint("    not found...")

    vw0.vprint("DONE.")

if __name__ == "__main__":
    import sys
    dest = sys.argv[1]
    print("loading Destination Workspace: %r" % dest)
    vw0 = vivisect.VivWorkspace()
    vw0.loadWorkspace(dest)

    for srcidx in range(2, len(sys.argv)):
        src = sys.argv[srcidx]
        subs = None
        # check if we tagged this as a different name than what the file suggests
        if ':' in src:
            src, subs_str = src.split(':', 1)
            subs = {key: val for key, val in [keyval.split('=') for keyval in subs_str.split(',')]}

        vw1 = vivisect.VivWorkspace()
        print("loading import Workspace: %r" % src)
        vw1.loadWorkspace(src)

        if not input("Merge Metadata from %r  ==> into ==> %r? Type YES to confirm: " %(src, dest)) == "YES":
            print("Skipping.")
            continue

        print("Merging...")
        immerge(vw0, vw1, subs)
        vw0.vprint("Saving Workspace...")
        vw0.saveWorkspace()




'''
syscall calling conventions:
    x86-32 [Free|Open|Net|DragonFly]BSD UNIX System Call convention:
    Parameters are passed on the stack. Push the parameters (last parameter pushed first) on to the stack. Then push an additional 32-bit of dummy data (Its not actually dummy data. refer to following link for more info) and then give a system call instruction int $0x80
    http://www.int80h.org/bsdasm/#default-calling-convention
'''



#!/usr/bin/python

#  toy rot13+... "rot13 + character location" so "s" (which rot's to "f") in the third char would be "h" and in the fourth char would be "i"
import sys
s = sys.argv[1]
p = 13
out=""
for i in s:
 nextchar = ord(i) + p
 if (nextchar > 124):
  nextchar -= 26
 out += chr(nextchar)
 p +=1

print out



#!/usr/bin/ipython  -i
import envi.memory as e_mem
import envi.archs.i386 as x86
import envi.archs.i386.emu as x86emu
from envi.archs.i386 import REG_EAX,REG_EBX,REG_ECX,REG_EDX,REG_ESI,REG_EDI,REG_EIP,REG_ESP,REG_EBP
import sys,struct


DEF_STACK_SIZE = 1024*1024
DEF_STACK_BASE = 0xc0000000
DEF_CODE_BASE = 0x8048100
a = x86.i386Module()

def opnum(num):
    return a.makeOpcode(struct.pack("<L", num))


def disass(bytes):
    offset = 0
    try:
        while True:
            op = a.makeOpcode(bytes, offset)
            nibble = bytes[offset:offset+len(op)]
            print "%8x\t%-20s\t%.20s"%(offset, nibble.encode('hex'), op)
            offset += len(op)
    except Exception, e:
        print "ERROR OCCURRED: %s" % repr(e)

class SuccessfulRunException(Exception): pass

class IEmulator(x86emu.IntelEmulator):
    def __init__(self, maps=[]):
        global a
        x86emu.IntelEmulator.__init__(self)
        self.imem_arch = a

        for mva,mperms,mnm,mbytes in maps:
            self.addMemoryMap(mva, mperms, mnm, mbytes)
            if mnm == '[stack]':
                self.setStackCounter(mva + len(mbytes) - 40)
                self.setRegister(REG_EBP, mva + len(mbytes) - 40)

        if len(maps):
            self.setProgramCounter(maps[0][0])

    def run(self, stepcount=None, tova=None):

        print self
        if stepcount != None:
            for i in xrange(stepcount):
                try:
                    self.stepi()
                    if tova is not None and self.getProgramCounter() == tova:
                        raise(SuccessfulRunException("Found VA"))
                except SuccessfulRunException, e:
                    print e
                    return
        else:
            while True:
                try:
                    self.stepi()
                    if tova is not None and self.getProgramCounter() == tova:
                        raise(SuccessfulRunException("Found VA"))
                except SuccessfulRunException, e:
                    print e
                    return


    def stepi(self):
        x86emu.IntelEmulator.stepi(self)
        print self

    def __str__(self):
        return self.getString()

    def getString(me, eachThread=False, width=8):
        output = []
        ebp= me.getRegister(REG_EBP)
        esp= me.getRegister(REG_ESP)
        eax= me.getRegister(REG_EAX)
        ebx= me.getRegister(REG_EBX)
        ecx= me.getRegister(REG_ECX)
        edx= me.getRegister(REG_EDX)
        esi= me.getRegister(REG_ESI)
        edi= me.getRegister(REG_EDI)
        eip= me.getProgramCounter() 
        
        output.append("\nesp: (%x)\n" % esp)
        try:
            output.append( me.XW(esp, 64, width) )
        except Exception, e:
            output.append("ERROR READING FROM MEMORY AT %x\n"%(esp) + repr(e))
        
        output.append( "\nebp-92: " + "(%x)\n" % esp )
        try:
            output.append(me.XW(ebp-92, 64, width))
        except Exception, e:
            output.append("ERROR READING FROM MEMORY AT %x\n"%(ebp-92) + repr(e))
        
        for i in ['eax','ecx','edx','ebx','esi','edi']:
            output.append("%s: %.08x\t"%(i,locals().get(i)))

        op = me.parseOpcode(eip)
        opers = []
        for oidx in range(len(op.opers)):
            try:
                opers.append("(%x)" % me.getOperAddr(op, oidx))
            except Exception,e:
                pass#print e
            opers.append(hex(me.getOperValue(op, oidx)))
        cmd = ' '.join(opers)
        output.append( "\neip: %.08x: %-30s\t\t\t%s"%(eip, op, cmd))
        output.append( "\n\t(%8x (%x): %d  %s)"%me.getMemoryMap(eip))
        return ''.join(output)

    def XW(tracer, address, length = 32, dwperline = 8):
        output = []
        for i in range(length):
            if (i % dwperline == 0):
                output.append( "%.08x:\t "%(address+(i*4)))
            try:
                bs = tracer.readMemory(address+(i*4),4)
                for x in range(4):
                    output.append("%.02x"%(ord(bs[x])))
            except:
                output.append("=-=-=-=-")

            if ((i+1) % dwperline == 0):
                output.append("\n")
            else:
                output.append("  ")
        return "".join(output)


def geti386Emulator(bytes, codebase=DEF_CODE_BASE, stackbase=DEF_STACK_BASE, stacksize=DEF_STACK_SIZE):
    maps = [(codebase, 5, 'program',  bytes),
            (stackbase-stacksize, 6, '[stack]', "D" * stacksize)]
    emu = IEmulator(maps)
    #emu.setStackCounter(stackbase - 4)
   
    return emu


if __name__ == '__main__':
    if len(sys.argv) > 1 and sys.argv[1] == "-o":
        sys.argv.pop(1)
        offset = int(sys.argv.pop(1))

    if len(sys.argv) > 1:
        print "Reading instructions from file %s"%sys.argv[1]
        inf = file(sys.argv[1], "rb")
    else:
        print "Reading instructions from stdin..."
        inf = sys.stdin

    bytes = inf.read()
    disass(bytes)




from idautils import *
from idaapi import *

ea = BeginEA()

symbols = {}
for funcea in Functions(SegStart(ea), SegEnd(ea)):
    functionName = GetFunctionName(funcea)
    functionStart = "0x%08x"%funcea
    functionEnd = "0x%08x"%FindFuncEnd(funcea)
    #print("%s : %s" % (functionName, functionStart))
    symbols[functionStart] = functionName


import pickle
pickle.dump(symbols, file('exportsyms.pickle', 'w'))

#print("%s : %s" % (functionName, functionStart))



def chopaddress(address):
    addy = address.strip(' \*$')
    try:
        if (addy[1] == 'x'):
            addy = addy[2:]
        if (addy[0] == '0'):
            addy = addy[1:]
    except:
        pass
    return addy
            


import os
import sys
import vivisect
import atlasutils.smartprint as asp
from IPython.display import display

dependencies = {}


def scanDirectory(dirpath, recurse=True):
    names = os.listdir(dirpath)
    for name in names:
        fname = os.sep.join([dirpath,name])
        if os.path.isfile(fname):
            dumpImports(fname)

        elif os.path.isdir(fname) and recurse:
            scanDirectory(fname)

def dumpImports(filename):
    vw = vivisect.VivWorkspace()
    try:
        vw.loadFromFile(filename)
    except:
        print "%s not known executable format" % filename
        return

    try:
        display(vw.filemeta)
        imports = vw.getImports()
        display(imports)

        for lva, lsz, ltp, name in imports:
            libsplit = name.split('.',1)
            if len(libsplit) == 1:
                print("IMPORT WITHOUT LIBRARY: %s" % name)
                lib = "unknown"
                sym = name
            else:
                lib, sym = libsplit

            symlist = dependencies.get(lib)
            if symlist == None:
                symlist = {}
                dependencies[lib] = symlist

            symentry = symlist.get(sym)
            if symentry == None:
                symentry = {}
                symlist[sym] = symentry

            symentry[filename] = symentry.get(filename, 0) + 1

    except Exception, e:
        print e



def main(argv):
    for dirname in argv[1:]:
        scanDirectory(dirname)

if __name__ == '__main__':
    main(sys.argv)



'''
scan through xrefs looking for VFTables, labeling them appropriately
'''

# TODO: VFTables
# TODO: VF Naming and Correlation
# TODO: Static Classes

def analyze(vw):
    # identify VFTables
    # cycle through XREFs of type REF_PTR
    for xrfr, xrto, xrtype, xrflag in vw.getXrefs(REF_PTR):
        # make sure the target is a pointer (possibly having valid pointers after??  
        ptr = vw.readMemoryPtr(xrto)
        if not vw.isValidPointer(ptr): continue
        if not vw.isExecutable(ptr): continue
        if not vw.isProbablyCode(ptr): continue

        # check for XREFs to that location
            # none should be REF_CODE
            # should be from instructions like "push" and "lea" and "mov" (x86-specific.  ugh)
            # should be moved into a reg-deref like [esi] or [ecx] or [eax] in one of the instructions...  ?? 

    # label as VFTables

    # label Virtual Functions (that aren't already named)


if 'vw' in globals().keys():
    analyze(vw)




#!/usr/bin/env python
import cmd
import sys
import time
import struct
import struct
import vtrace
import traceback
import collections

import envi
import envi.exc as e_exc
import envi.memory as e_m
import envi.expression as e_expr
import envi.memcanvas as e_memcanvas

import envi.archs.i386 as e_i386
import envi.archs.amd64 as e_amd64

import vivisect
import visgraph.pathcore as vg_path
from binascii import hexlify, unhexlify

from atlasutils.errno import *


SNAP_NORM = 0
SNAP_CAP = 1
SNAP_DIFF = 2
SNAP_SWAP = 3

PEBSZ = 4096
TEBSZ = 4096

def parseExpression(emu, expr, lcls={}):
    '''
    localized updated expression parser for the emulator at any state
    '''
    if hasattr(emu, 'vw'):
        lcls.update(emu.vw.getExpressionLocals())
    lcls.update(emu.getRegisters())
    if isinstance(emu, vtrace.Trace):
        lcls.update(emu.getRegisterContext().getRegisters())
    return e_expr.evaluate(expr, lcls)

import vivisect.impemu.monitor as v_i_monitor
class TraceMonitor(v_i_monitor.AnalysisMonitor):
    def __init__(self, traces=None):
        if traces is None:
            traces = {}
        self.traces = traces

    def prehook(self, emu, op, starteip):
        tdata = self.traces.get(starteip)
        if tdata is None:
            return

        try:
            print(repr(eval(tdata)))
        except Exception as e:
            print("TraceMonitor ERROR at 0x%x: %r" % (starteip, e))

testemu = None
call_handlers = {}
def runStep(emu, maxstep=1000000, follow=True, showafter=True, runTil=None, pause=True, silent=False, finish=0, tracedict=None, verbose=False, guiFuncGraphName=None):
    global testemu

    if testemu is None or testemu.emu != emu:
        testemu = TestEmulator(emu, verbose=verbose, guiFuncGraphName=guiFuncGraphName)
        testemu.call_handlers.update(call_handlers)
    
    testemu.runStep(maxstep=maxstep, follow=follow, showafter=showafter, runTil=runTil, pause=pause, silent=silent, finish=finish, tracedict=tracedict)


def readString(emu, va, CHUNK=50):
    off = 0
    out = [ emu.readMemory(va + off, CHUNK) ]
    while b'\0' not in out[-1]:
        off += CHUNK
        data = emu.readMemory(va + off, CHUNK)
        out.append(data)

    data = b''.join(out)

    return data[:data.find(b'\0')]

prehilite = '\x1b[7m'
posthilite = '\x1b[27m'
def compare(data1, data2):
    size = (len(data1), len(data2))[len(data1) > len(data2)]

    out1 = []
    out2 = []
    lastres = True
    for x in range(size):
        if data1[x] != data2[x]:
            if lastres:
                out1.append(prehilite)
                out2.append(prehilite)
            lastres = False
        else:
            if not lastres:
                out1.append(posthilite)
                out2.append(posthilite)
            lastres = True

        out1.append(data1[x:x+1].hex())
        out2.append(data2[x:x+1].hex())
   
    if len(data1) > len(data2):
        out1.append(data1[x:].hex())
    elif len(data1) > len(data2):
        out2.append(data2[x:].hex())
    
    if not lastres:
        out1.append(posthilite)
        out2.append(posthilite)

    print(''.join(out1))
    print(''.join(out2))


#######  replacement functions.  can set these in TestEmulator().call_handlers 
#######  to execute these in python instead of the supporting library
#######  can also be run from runStep() ui to execute the replacement function
def getMSCallConv(emu, tva=None, wintel32pref='stdcall'):
    if hasattr(emu, 'vw') and emu.vw is not None:
        ccname = None
        tloc = emu.vw.getLocation(tva)
        if tloc is not None:
            tlva, tlsz, tltype, tltinfo = tloc
            if tltype == vivisect.LOC_IMPORT:
                impapi = emu.vw.getImpApi(tltinfo)
                if impapi is not None:
                    rettyp, _, ccname, realname, args = impapi

        if ccname is None:
            if emu.psize == 4: # and emu._arch ????:
                ccname = wintel32pref
            else:
                ccname = emu.vw.getMeta('DefaultCall')

        cconv = emu.getCallingConvention(ccname)
        return ccname, cconv

    return emu.getCallingConventions()[0]

def getLibcCallConv(emu):
    if hasattr(emu, 'vw') and emu.vw is not None:
        ccname = emu.vw.getMeta('DefaultCall')
        cconv = emu.getCallingConvention(ccname)
        return ccname, cconv

    return emu.getCallingConventions()[0]

def memset(emu, op=None):
    ccname, cconv = getLibcCallConv(emu)
    dest, char, count = cconv.getCallArgs(emu, 3)

    data = ('%c' % char) * count
    emu.writeMemory(dest, data)
    print(data)
    cconv.execCallReturn(emu, 0, 0)
    return data

def memcpy(emu, op=None):
    ccname, cconv = getLibcCallConv(emu)
    dest, src, length = cconv.getCallArgs(emu, 3)
    data = emu.readMemory(src, length)
    emu.writeMemory(dest, data)
    print(data)
    cconv.execCallReturn(emu, dest, 0)

    return data

def strncpy(emu, op=None):
    ccname, cconv = getLibcCallConv(emu)
    dest, src, length = cconv.getCallArgs(emu, 3)
    data = emu.readMemory(src, length)
    nulloc = data.find(b'\0')
    if nulloc != -1:
        data = data[:nulloc]
    emu.writeMemory(dest, data)
    print(data)
    cconv.execCallReturn(emu, dest, 0)
    return data

def strncpy_s(emu, op=None):
    ccname, cconv = getLibcCallConv(emu)
    dest, destsz, src, length = cconv.getCallArgs(emu, 4)
    
    length = min(destsz, length)
    data = emu.readMemory(src, length)

    nulloc = data.find(b'\0')
    if nulloc != -1:
        data = data[:nulloc]

    data += b'\0'

    # check to see if need to return an error??

    emu.writeMemory(dest, data)
    print(data)
    cconv.execCallReturn(emu, dest, 0)
    return data

def strcpy(emu, op=None):
    ccname, cconv = getLibcCallConv(emu)
    dest, src = cconv.getCallArgs(emu, 2)
    data = readString(emu, src) + b'\0'
    emu.writeMemory(dest, data)
    print(data)
    cconv.execCallReturn(emu, dest, 0)
    return data

def strcat(emu, op=None):
    ccname, cconv = getLibcCallConv(emu)
    start, second = cconv.getCallArgs(emu, 2)
    initial = readString(emu, start)
    data = readString(emu, second)
    emu.writeMemory(start + len(initial) + b'\0', data)
    print(initial+data)
    cconv.execCallReturn(emu, dest, 0)
    return initial+data

def strncat(emu, op=None):
    ccname, cconv = getLibcCallConv(emu)
    start, second, max2 = cconv.getCallArgs(emu, 3)
    initial = readString(emu, start)
    data = readString(emu, second)[:max2]
    
    emu.writeMemory(start + len(initial), data)
    print(initial+data)
    cconv.execCallReturn(emu, dest, 0)
    return initial+data

def strncat_s(emu, op=None):
    ccname, cconv = getLibcCallConv(emu)
    start, destsz, second, max2 = cconv.getCallArgs(emu, 4)
    initial = readString(emu, start)
    data = readString(emu, second)[:max2]

    initiallen = len(initial)
    writelen = destsz - initiallen
    
    emu.writeMemory(start + initiallen, data[:writelen])
    print(initial+data)
    cconv.execCallReturn(emu, 0, 0)
    return initial+data

def strlen(emu, op=None):
    ccname, cconv = getLibcCallConv(emu)
    start, = cconv.getCallArgs(emu, 1)
    data = readString(emu, start)
    print(len(data))
    cconv.execCallReturn(emu, len(data), 0)
    return len(data)

def strcmp(emu, op=None):
    ccname, cconv = getLibcCallConv(emu)
    start1, start2 = cconv.getCallArgs(emu, 2)
    data1 = readString(emu, start1)
    data2 = readString(emu, start2)
    data1len = len(data1)
    data2len = len(data2)
    failed = False

    if data1len != data2len:
        failed = True
        data1 += '\0'
        data2 += '\0'

    for idx in min(data1len, data2len):
        if data1[idx] != data2[idx]:
            failed = True
            break
    
    retval = data2[idx] - data1[idx]
    if failed:
        print("strcmp failed: %d" % retval)

    cconv.execCallReturn(emu, retval, 0)
    return retval


PAGE_SIZE = 1 << 12
PAGE_NMASK = PAGE_SIZE - 1
PAGE_MASK = ~PAGE_NMASK
CHUNK_SIZE = 1 << 4
CHUNK_NMASK = CHUNK_SIZE - 1
CHUNK_MASK = ~CHUNK_NMASK


class EmuHeap:
    def __init__(self, emu, size=10*1024, startingpoint=0x20000000):
        self.emu = emu
        self.size = size

        mmap = '\0' * size

        #heapbase = emu.findFreeMemoryBlock(size, startingpoint)
        #if not emu.getMemoryMap(heapbase):
        #    emu.addMemoryMap(heapbase, 6, 'heap_%x'%heapbase, b'\0'*size)
        heapbase = emu.allocateMemory(size, 6, startingpoint, 'heap')    
        self.ptr = heapbase
        self.tracker = {}
        self.freed = {}

    def malloc(self, size):
        size += CHUNK_NMASK
        size &= CHUNK_MASK
        chunk = self.ptr
        self.ptr += size

        self.tracker[chunk] = (size, self.emu.getProgramCounter())
        return chunk

    def realloc(self, chunk, size):
        if chunk not in self.tracker:
            return 0

        newchunk = self.malloc(size)
        oldsize = self.tracker.get(chunk)
        self.emu.writeMemory(newchunk, self.emu.readMemory(chunk, oldsize))

        return newchunk

    def free(self, addr):
        self.freed[addr] = self.emu.getProgramCounter()

    def dump(self):
        out = []
        for baseva, (size, allocpc) in list(self.tracker.items()):
            data = self.emu.readMemory(baseva, size)
            out.append("[0x%x:0x%x]: %r (0x%x)" % (baseva, size, data.hex(), allocpc))

        return '\n'.join(out)


def getHeap(emu, initial_size=None):
    '''
    Returns a Heap Object.
    If one is not currently created in the emu (stored in emu metadata)
    one is created.  If initial_size is not None, that value is used,
    otherwise the default is used.
    '''
    heap = emu.getMeta('Heap')
    if heap is None:
        if initial_size is not None:
            heap = EmuHeap(emu, initial_size)
        else:
            heap = EmuHeap(emu)
        emu.setMeta('Heap', heap)

    return heap


#### Win32 helper functions
def Sleep(emu, op=None):
    ccname, cconv = getMSCallConv(emu, op.va)
    dwMS = cconv.getCallArgs(emu, 1)
    print("Sleep: dwMillisectonds: %d" % (dwMS))
    # calling getHeap initializes a heap.  we can cheat for now.  we may need to initialize new heaps here
    time.sleep(dwMS/1000)
    cconv.execCallReturn(emu, 0, 1)

def HeapCreate(emu, op=None):
    ccname, cconv = getMSCallConv(emu, op.va)
    opts, initsz, maxsz = cconv.getCallArgs(emu, 3)
    print("HeapCreate: flOptions: 0x%x dwInitialSize: 0x%x, dwMaxSize" % (opts, initsz, maxsz))
    # calling getHeap initializes a heap.  we can cheat for now.  we may need to initialize new heaps here
    cconv.execCallReturn(emu, emu.setVivTaint('MSHeap', op.va), 3)

def HeapDestroy(emu, op=None):
    ccname, cconv = getMSCallConv(emu, op.va)
    heapHandle = cconv.getCallArgs(emu, 1)
    print("HeapDestroy: 0x%x" % heapHandle)
    # calling getHeap initializes a heap.  we can cheat for now.  we may need to initialize new heaps here
    cconv.execCallReturn(emu, heapHandle, 1)

def HeapAlloc(emu, op=None):
    '''
    This is a functional heap implementation, not intended to behave in any way like 
    the MS heap or any other heap impls available.  It gives you a chunk of memory so
    the program you're playing with keeps going.
    That's it.
    dwflags is ignored completely.
    '''
    ccname, cconv = getMSCallConv(emu, op.va)
    hheap, dwflags, size = cconv.getCallArgs(emu, 3)

    heap = getHeap(emu)
    allocated_ptr = heap.malloc(size)
    print("malloc(0x%x)  => 0x%x" % (size, allocated_ptr))
    cconv.execCallReturn(emu, allocated_ptr, 3)

def HeapFree(emu, op=None):
    ccname, cconv = getMSCallConv(emu, op.va)
    hheap, dwflags, va = cconv.getCallArgs(emu, 3)
    print("FREE: 0x%x" % va)
    cconv.execCallReturn(emu, va, 3)

def HeapReAlloc(emu, op=None):
    ccname, cconv = getMSCallConv(emu, op.va)
    hheap, dwflags, existptr, size, = cconv.getCallArgs(emu, 4)

    heap = getHeap(emu)
    allocated_ptr = heap.realloc(existptr, size)
    cconv.execCallReturn(emu, allocated_ptr, 4)

critical_sections = collections.defaultdict(list)
def InitializeCriticalSection(emu, op=None):
    global critical_sections
    ccname, cconv = getMSCallConv(emu, op.va)
    lpCriticalSection, = cconv.getCallArgs(emu, 1)
    critical_sections[lpCriticalSection].append(op.va)
    # do absolutely nothing
    cconv.execCallReturn(emu, 0, 0)

def EnterCriticalSection(emu, op=None):
    global critical_sections
    ccname, cconv = getMSCallConv(emu, op.va)
    lpCriticalPointer, = cconv.getCallArgs(emu, 1)
    critical_sections[lpCriticalPointer].append(op.va)
    # do absolutely nothing
    cconv.execCallReturn(emu, 0, 1)

def LeaveCriticalSection(emu, op=None):
    global critical_sections
    ccname, cconv = getMSCallConv(emu, op.va)
    lpCriticalPointer, = cconv.getCallArgs(emu, 1)
    critical_sections[lpCriticalPointer].append(op.va)
    # do absolutely nothing

    cconv.execCallReturn(emu, 0, 1)

last_error = 0
def GetLastError(emu, op=None):
    global last_error
    ccname, cconv = getMSCallConv(emu, op.va)
    cconv.execCallReturn(emu, last_error, 0)

def SetLastError(emu, op=None):
    global last_error
    ccname, cconv = getMSCallConv(emu, op.va)
    last_error, = cconv.getCallArgs(emu, 1)
    cconv.execCallReturn(emu, 0, 1)

def GetCurrentThreadId(emu, op=None):
    ccname, cconv = getMSCallConv(emu, op.va)
    cconv.execCallReturn(emu, 0x31337, 0)

# TODO: wrap this into a TLS object and strap it into the emulator like we do with the Heap
tls_idxs = []
tls_next_idx = 100
tls_data = collections.defaultdict(list)

def rtTlsAlloc():
    global tls_idxs, tls_next_idx, tls_data
    idx = tls_next_idx
    tls_idxs.append(idx)
    tls_next_idx+= 1
    return idx

def rtTlsGetValue(slot):
    '''
    Returns None if nothing exists. By design.
    '''
    global tls_data
    if len(tls_data[slot]):
        return tls_data[slot][-1]

    print("rtTlsGetValue(%d) returning None, sorry..." % slot)


def rtTlsSetValue(slot, data):
    global tls_data
    tls_data[slot].append(data)
    return 1


def TlsAlloc(emu, op=None):
    # should we track this in the emulator?
    ccname, cconv = getMSCallConv(emu, op.va)
    cconv.execCallReturn(emu, rtTlsAlloc(), 0)

def TlsGetValue(emu, op=None):
    global tls_data
    ccname, cconv = getMSCallConv(emu, op.va)
    slot, = cconv.getCallArgs(emu, 1)

    tlsval = rtTlsGetValue(slot)
    print("TlsGetValue(%d): found %r" % (slot, tlsval))

    if tlsval is None:  # do this here since we have an op and emu already, and it makes sense
        tlsval = emu.setVivTaint('TlsGetValue::Slot at 0x%x' % op.va, slot)
        rtTlsSetValue(slot, tlsval)

    cconv.execCallReturn(emu, tlsval, 1)

def TlsSetValue(emu, op=None):
    global tls_data
    ccname, cconv = getMSCallConv(emu, op.va)
    slot, data = cconv.getCallArgs(emu, 2)
    cconv.execCallReturn(emu, rtTlsSetValue(slot, data), 2)


def CompareStringEx(emu, op=None):
    ccname, cconv = getMSCallConv(emu, op.va)
    lpLocaleName, dwCmpFlags, lpString1, cchCount1, lpString2, cchCount2, lpVersionInfo, \
            lpRsrvd, lParam = cconv.getCallArgs(emu, 9)
    result = doWin32StringCompare(emu, op, Locale, dwCmpFlags, lpString1, cchCount1, \
            lpString2, cchCount2, lpVersionInfo, lpRsrvd, lParam, charsize=2)

    cconv.execCallReturn(emu, result, 9)

def CompareStringW(emu, op=None):
    ccname, cconv = getMSCallConv(emu, op.va)
    Locale, dwCmpFlags, lpString1, cchCount1, lpString2, cchCount2 = cconv.getCallArgs(emu, 6)
    result = doWin32StringCompare(emu, op, Locale, dwCmpFlags, lpString1, cchCount1, \
            lpString2, cchCount2, 0,0,0, charsize=2)

    cconv.execCallReturn(emu, result, 6)

def CompareStringA(emu, op=None):
    ccname, cconv = getMSCallConv(emu, op.va)
    Locale, dwCmpFlags, lpString1, cchCount1, lpString2, cchCount2 = cconv.getCallArgs(emu, 6)
    result = doWin32StringCompare(emu, op, Locale, dwCmpFlags, lpString1, cchCount1, \
            lpString2, cchCount2, 0,0,0, charsize=2)

    cconv.execCallReturn(emu, result, 6)

CSTR_FAILURE = 0
CSTR_LESS_THAN = 1
CSTR_EQUAL = 2
CSTR_GREATER_THAN = 3

def doWin32StringCompare(emu, op, \
        Locale, dwCmpFlags, lpString1, cchCount1, lpString2, cchCount2, \
        lpVersionInfo, lpReserved, lParam, charsize=1):

    if dwCmpFlags:
        print("CompareStringEx with flags %x, unsupported." % dwCmpFlags)

    idx = 0
    result = 0
    while True:
        if (cchCount1 != -1 and idx > cchCount1):
            if cchCount1 == cchCount2:
                return CSTR_EQUAL
            if cchCount2 == -1 and val2[0] == 0:
                return CSTR_EQUAL
            return CSTR_GREATER # ? if str1 is done and str2 isn't?

        if (cchCount2 != -1 and idx > cchCount2):
            if cchCount1 == cchCount2:
                return CSTR_EQUAL
            if cchCount1 == -1 and val1[0] == 0:
                return CSTR_EQUAL
            return CSTR_LESS_THAN   # ? if str2 is done and str1 isn't?

        val1 = emu.readMemory(lpString1 + idx, charsize)
        val2 = emu.readMemory(lpString2 + idx, charsize)
        # do any conversions necessary (skipping for now, i'm feeling lucky)

        # do comparison.  this version is cheating:
        for x in range(charsize):
            val1part = val1[x]
            val2part = val2[x]
            result = val1part - val2part
            if result:
                return result + 2   # MS likes 1,2,3 where 0 is failure

        idx += charsize

    return CSTR_FAILURE

def vsnprintf(emu, op=None):
    '''
    Simplistic, but good enough for most government work...
    '''
    stackDump(emu)
    ccname, cconv = getMSCallConv(emu, op.va, 'cdecl')
    s, n, fmt, args = cconv.getCallArgs(emu, 4)
    outfmt = emu.readMemString(fmt)

    off = 0
    arglist = []
    if b'%' in outfmt:
        bits = emu.readMemoryPtr(args + off)
        if emu.getMemoryMap(bits):
            arglist.append(emu.readMemString(bits))
        else:
            arglist.append(bits)
        off += 4

    while True:
        #print(outfmt, tuple(arglist))
        try:
            out = outfmt % tuple(arglist)
            break
        except TypeError:
            bits = emu.readMemoryPtr(args + off)
            if emu.getMemoryMap(bits):
                arglist.append(emu.readMemString(bits))
            else:
                arglist.append(bits)
            off += 4

    emu.writeMemory(s, out[:n])
    result = len(out)

    input("vsnprintf: %r" % out)
    cconv.execCallReturn(emu, result, 4)
    
def getenv_s(emu, op=None):
    '''
    Get Environment Variables....
    Since we don't have any environment variables, 
    '''
    ccname, cconv = getMSCallConv(emu, op.va)
    pRetVal, buff, numElem, varnameptr = cconv.getCallArgs(emu, 4)

    # grab some fake Environment variable string...
    temu = emu.temu
    vw = temu.vw
    varname = temu.emu.readMemString(varnameptr)
    print("getenv_s(0x%x, 0x%x, 0x%x, %r)" % (pRetVal, buff, numElem, varname))

    os = vw.getMeta('Platform')
    if os in ('windows', 'winkern'):
        # case-insensitive... everything must be UPPERs
        varname = varname.upper()

    # lookup the env var:
    val = temu.environment.get(varname)

    # figure out how to return
    if not pRetVal:
        # can't actually put any meaningful response.
        print("no pRetVal! EINVAL")
        result = EINVAL

    elif val is None:
        # throw a tantrum, couldn't find it mom!
        emu.writeMemoryPtr(pRetVal, 0)
        result = EINVAL
        print("couldn't find %r! EINVAL" % varname)

    else:
        # found it...
        if not buff or numElem == 0:
            # just return the size requirements
            emu.writeMemoryPtr(pRetVal, len(val))

        else:
            # write out as much as we can...
            emu.writeMemory(buff, val[:numElem])
        result = 0

    cconv.execCallReturn(emu, result, 0)

INVALID_FILE_ATTRIBUTES = -1
def GetFileAttributesA(emu, op=None):
    '''
    Return attributes of a file or directory.
    '''
    ccname, cconv = getMSCallConv(emu, op.va)
    lpFileName, = cconv.getCallArgs(emu, 1)
    filename = emu.readMemString(lpFileName).upper()

    fsentry = emu.temu.fs.get(filename)
    if not fsentry:
        result = INVALID_FILE_ATTRIBUTES

    else:
        lstat, fdata = fsentry
        result = lstat

    cconv.execCallReturn(emu, result, 1)



#### posix function helpers
def malloc(emu, op=None):
    '''
    emulator hook for malloc calls
    '''
    ccname, cconv = getLibcCallConv(emu)
    size, = cconv.getCallArgs(emu, 1)

    heap = getHeap(emu)
    allocated_ptr = heap.malloc(size)
    print("malloc(0x%x)  => 0x%x" % (size, allocated_ptr))

    cconv.execCallReturn(emu, allocated_ptr, 0)

def calloc(emu, op=None):
    '''
    emulator hook for malloc calls
    '''
    ccname, cconv = getLibcCallConv(emu)
    elements, size = cconv.getCallArgs(emu, 2)

    heap = getHeap(emu)
    allocated_ptr = heap.malloc(size * elements)
    print("calloc(0x%x, 0x%x)  => 0x%x" % (elements, size, allocated_ptr))

    cconv.execCallReturn(emu, allocated_ptr, 0)

def free(emu, op=None):
    '''
    emulator hook for free calls
    '''
    ccname, cconv = getLibcCallConv(emu)
    va, = cconv.getCallArgs(emu, 1)
    heap = getHeap(emu)
    heap.free(va)
    print("FREE: 0x%x" % va)
    cconv.execCallReturn(emu, 0, 0)

def realloc(emu, op=None):
    '''
    emulator hook for realloc calls
    '''
    ccname, cconv = getLibcCallConv(emu)
    existptr, size, = cconv.getCallArgs(emu, 2)

    heap = getHeap(emu)
    allocated_ptr = heap.realloc(existptr, size)
    cconv.execCallReturn(emu, allocated_ptr, 0)

def ret0(emu, op):
    '''
    emulator hook to just return 0
    '''
    ccname, cconv = getLibcCallConv(emu)
    cconv.execCallReturn(emu, 0, 0)

def ret1(emu, op):
    '''
    emulator hook to just return 1
    '''
    ccname, cconv = getLibcCallConv(emu)
    cconv.execCallReturn(emu, 1, 0)

def retneg1(emu, op):
    '''
    emulator hook to just return -1
    '''
    ccname, cconv = getLibcCallConv(emu)
    cconv.execCallReturn(emu, -1, 0)


def syslog(emu, op=None):
    '''
    emulator hook for calls to syslog
    '''
    ccname, cconv = getLibcCallConv(emu)
    loglvl, strlength = cconv.getCallArgs(emu, 2)
    string = readString(emu, strlength)
    count = string.count('%')
    neg2 = string.count('%%')
    count -= (2*neg2)

    args = cconv.getCallArgs(emu, count+2)[2:]
    outstring = string % args
    print("SYSLOG(%d): %s" % (loglvl, outstring))
    for s in args:
        if emu.isValidPointer(s):
            print("\t" + readString(emu, s))
    cconv.execCallReturn(emu, 0, 0)

def nop(emu, op=None):
    pass

import_map = {
        '*.syslog': syslog,
        '*.malloc': malloc,
        '*.calloc': calloc,
        '*.free': free,
        '*.realloc': realloc,
        '*.strlen': strlen,
        '*.strcmp': strcmp,
        '*.strcat': strcat,
        '*.strcpy': strcpy,
        '*.strncpy': strncpy,
        '*.memcpy': memcpy,
        '*.memset': memset,
        'kernel32.Sleep': Sleep,
        'kernel32.HeapAlloc': HeapAlloc,
        'kernel32.HeapFree': HeapFree,
        'kernel32.HeapReAlloc': HeapReAlloc,
        'kernel32.HeapDestroy': HeapDestroy,
        'kernel32.HeapCreate': HeapCreate,     # malloc takes care of this
        'kernel32.InitializeCriticalSection': InitializeCriticalSection,
        'kernel32.EnterCriticalSection': EnterCriticalSection,
        'kernel32.LeaveCriticalSection': LeaveCriticalSection,
        'kernel32.GetLastError': GetLastError,
        'kernel32.SetLastError': SetLastError,
        'kernel32.TlsAlloc': TlsAlloc,
        'kernel32.TlsGetValue': TlsGetValue,
        'kernel32.TlsSetValue': TlsSetValue,
        'kernel32.GetCurrentThreadId': GetCurrentThreadId,
        'kernel32.CompareStringW': CompareStringW,
        'kernel32.CompareStringA': CompareStringA,
        'kernel32.GetFileAttributesA': GetFileAttributesA,
        'ntdll._vsnprintf': vsnprintf,
        'msvcr100.getenv_s': getenv_s,
        'msvcr100.calloc': calloc,
        'msvcr100.strncpy_s': strncpy_s,
        'msvcr100.strncat_s': strncat_s,
        'msvcr100.free': free,
        }


class win32const:
    FILE_ATTRIBUTE_ARCHIVE = 32 #(0x20) A file or directory that is an archive file or directory. Applications typically use this attribute to mark files for backup or removal .
    FILE_ATTRIBUTE_COMPRESSED = 2048 #(0x800) A file or directory that is compressed. For a file, all of the data in the file is compressed. For a directory, compression is the default for newly created files and subdirectories.
    FILE_ATTRIBUTE_DEVICE = 64 #(0x40) This value is reserved for system use.
    FILE_ATTRIBUTE_DIRECTORY = 16 #(0x10) The handle that identifies a directory.
    FILE_ATTRIBUTE_ENCRYPTED = 16384 #(0x4000) A file or directory that is encrypted. For a file, all data streams in the file are encrypted. For a directory, encryption is the default for newly created files and subdirectories.
    FILE_ATTRIBUTE_HIDDEN = 2 #(0x2) The file or directory is hidden. It is not included in an ordinary directory listing.
    FILE_ATTRIBUTE_INTEGRITY_STREAM = 32768 #(0x8000) The directory or user data stream is configured with integrity (only supported on ReFS volumes). It is not included in an ordinary directory listing. The integrity setting persists with the file if it's renamed. If a file is copied the destination file will have integrity set if either the source file or destination directory have integrity set.  Windows Server 2008 R2, Windows 7, Windows Server 2008, Windows Vista, Windows Server 2003 and Windows XP: This flag is not supported until Windows Server 2012.
    FILE_ATTRIBUTE_NORMAL = 128 #(0x80) A file that does not have other attributes set. This attribute is valid only when used alone.
    FILE_ATTRIBUTE_NOT_CONTENT_INDEXED = 8192 #(0x2000) The file or directory is not to be indexed by the content indexing service.
    FILE_ATTRIBUTE_NO_SCRUB_DATA = 131072 #(0x20000) The user data stream not to be read by the background data integrity scanner (AKA scrubber). When set on a directory it only provides inheritance. This flag is only supported on Storage Spaces and ReFS volumes. It is not included in an ordinary directory listing.  Windows Server 2008 R2, Windows 7, Windows Server 2008, Windows Vista, Windows Server 2003 and Windows XP: This flag is not supported until Windows 8 and Windows Server 2012.
    FILE_ATTRIBUTE_OFFLINE = 4096 #(0x1000) The data of a file is not available immediately. This attribute indicates that the file data is physically moved to offline storage. This attribute is used by Remote Storage, which is the hierarchical storage management software. Applications should not arbitrarily change this attribute.
    FILE_ATTRIBUTE_READONLY = 1 #(0x1) A file that is read-only. Applications can read the file, but cannot write to it or delete it. This attribute is not honored on directories. For more information, see You cannot view or change the Read-only or the System attributes of folders in Windows Server 2003, in Windows XP, in Windows Vista or in Windows 7.
    FILE_ATTRIBUTE_RECALL_ON_DATA_ACCESS = 4194304 #(0x400000) When this attribute is set, it means that the file or directory is not fully present locally. For a file that means that not all of its data is on local storage (e.g. it may be sparse with some data still in remote storage). For a directory it means that some of the directory contents are being virtualized from another location. Reading the file / enumerating the directory will be more expensive than normal, e.g. it will cause at least some of the file/directory content to be fetched from a remote store. Only kernel-mode callers can set this bit.
    FILE_ATTRIBUTE_RECALL_ON_OPEN = 262144 #(0x40000) This attribute only appears in directory enumeration classes (FILE_DIRECTORY_INFORMATION, FILE_BOTH_DIR_INFORMATION, etc.). When this attribute is set, it means that the file or directory has no physical representation on the local system; the item is virtual. Opening the item will be more expensive than normal, e.g. it will cause at least some of it to be fetched from a remote store.
    FILE_ATTRIBUTE_REPARSE_POINT = 1024 #(0x400) A file or directory that has an associated reparse point, or a file that is a symbolic link.
    FILE_ATTRIBUTE_SPARSE_FILE = 512 #(0x200) A file that is a sparse file.
    FILE_ATTRIBUTE_SYSTEM = 4 #(0x4) A file or directory that the operating system uses a part of, or uses exclusively.
    FILE_ATTRIBUTE_TEMPORARY = 256 #(0x100) A file that is being used for temporary storage. File systems avoid writing data back to mass storage if sufficient cache memory is available, because typically, an application deletes a temporary file after the handle is closed. In that scenario, the system can entirely avoid writing the data. Otherwise, the data is written after the handle is closed.
    FILE_ATTRIBUTE_VIRTUAL = 65536 #(0x10000) This value is reserved for system use.

#### SYSENTER/SYSCALL helpers
class SystemCallNotImplemented(Exception):
    def __init__(self, callnum, emu, op):
        Exception.__init__(self)
        self.callnum = callnum
        self.emu = emu
        self.op = op

    def __repr__(self):
        return "SystemCall 0x%x (%d) not implemented at 0x%x: %r" % (self.callnum, self.op.va, self.op)

class WinKernel(dict):
    def __init__(self, emu, vermaj=6, vermin=1, arch='i386', syswow=False):
        dict.__init__(self)
        self.emu = emu

        if syswow:
            arch = 'wow64'

        self.modbase = 'vstruct.defs.windows.win_%s_%s_%s' % (vermaj, vermin, arch)
        self.win32k = None
        self.ntdll = None
        self.ntoskrnl = None
        try:
            self.win32k = __import__(self.modbase + '.win32k', {}, {}, 1)
            self.ntdll = __import__(self.modbase + '.ntdll', {}, {}, 1)
            self.ntoskrnl = __import__(self.modbase + '.ntoskrnl', {}, {}, 1)
        except ImportError as e:
            print("error importing VStructs for Windows %d.%d_%s: %r" % (vermaj, vermin, arch, e))
        
        # setup key files db here
        self['fs'] = collections.defaultdict(dict)    # perhaps create file objects, for now this.
        self['fhandles'] = {}   # store a connection between a handle and a member of 'fs'

        # actual syscall handlers
        self.win_syscalls = {    # worked up on Win7-32
            0xd9: self.sys_win_NtQueryAttributesFile,  # ntdll.ntQueryAttributesFile
            0xdc: self.sys_win_DbgQueryDebugFilterState,  # ntdll.DbgQueryDebugFilterState
            0xb3: self.sys_win_NtOpenFile,
            0x54: self.sys_win_NtCreateSection,
            0xa8: self.sys_win_MapViewOfSection,
            }


    def op_sysenter(self, emu, op):
        # handle select Windows syscalls
        callnum = emu.getRegister(0)
        syscall = self.win_syscalls.get(callnum)
        if syscall is not None:
            syscall(emu, op)

        else:
            raise SystemCallNotImplemented(callnum, emu, op)

    def sys_win_DbgQueryDebugFilterState(self, emu, op):
        stackDump(emu)
        sp = emu.getStackCounter()
        arg0 = emu.readMemoryPtr(sp + emu.psize)    # second RET
        arg1 = emu.readMemoryPtr(sp + (2*emu.psize))
        arg2 = emu.readMemoryPtr(sp + (3*emu.psize))

        print("ntDbgQueryDebugFilterState( 0x%x, 0x%x )" % (arg1, arg2))
        # for now
        retval = 0
        emu.setRegister(0, retval)

    def getWinAbsTime(self, ts_since_unix_epoch):
        return (11644473600 + ts_since_unix_epoch) * 10000000

    def getUnixTime(self, ts_since_win_epoch):
        return (ts_since_win_epoch / 10000000) - 11644473600

    def parseUnicodeString(self, emu, addr):
        UNICODE_STRING = self.ntdll.UNICODE_STRING()
        UNICODE_STRING.vsParse(emu.readMemory(addr, len(UNICODE_STRING)))
        return UNICODE_STRING


    def sys_win_NtQueryAttributesFile(self, emu, op):
        stackDump(emu)
        sp = emu.getStackCounter()
        arg0 = emu.readMemoryPtr(sp + emu.psize)
        arg1 = emu.readMemoryPtr(sp + (2*emu.psize))
        arg2 = emu.readMemoryPtr(sp + (3*emu.psize))

        print("ntQueryAttributesFile( 0x%x, 0x%x )" % (arg1, arg2))

        length, rootdir, objname, attrib, secdesc, secqos = \
                emu.readMemoryFormat(arg1, "<IPPIPP")

        if length > 40:
            raise Exception("NtQueryAttributesFile.OBJECT_ATTRIBUTES length: 0x%x (wrong pointer?)" % length)

        fullpathstruct = self.parseUnicodeString(emu, objname)
        fullpath = emu.readMemory(fullpathstruct.Buffer, fullpathstruct.Length)

        if rootdir != 0:
            rootdirstruct = self.parseUnicodeString(emu, rootdir)
            fullpath = emu.readMemory(rootdirstruct.Buffer, rootdirstruct.Length) + fullpath

        print("FullPath: %r" % fullpath)
        # work in ROOTPATH here... right now, just fake
        f = self['fs'][fullpath]
        f['attribmask'] = attrib
        f['secqosptr'] = secqos
        f['secdescptr'] = secdesc
        # need to check the file 

        # FAKE NEWS!
        WriteTime = int(self.getWinAbsTime(time.time()))
        ChangeTime = int(self.getWinAbsTime(time.time()))
        AccessTime = int(self.getWinAbsTime(time.time()))
        CreationTime = int(self.getWinAbsTime(time.time()))
        Attributes = attrib

        print("len:%x rootdir:%x objname:%x attrib:%x secdesc:%x secqos:%x" %(length, rootdir, objname, attrib, secdesc, secqos))
        
        # now we need to write the output data!
        emu.writeMemoryFormat(arg2, '<QQQQI', CreationTime, AccessTime, WriteTime, ChangeTime, Attributes)

        import envi.interactive as ei; ei.dbg_interact(locals(), globals())

        # for now
        retval = 0  # STATUS_SUCCESS
        emu.setRegister(0, retval)

    def sys_win_NtOpenFile(self, emu, op):
        stackDump(emu)
        sp = emu.getStackCounter()
        arg0 = emu.readMemoryPtr(sp + emu.psize)
        arg1 = emu.readMemoryPtr(sp + (2*emu.psize))    # out: FileHandle
        arg2 = emu.readMemoryPtr(sp + (3*emu.psize))    # in: DesiredAccess
        arg3 = emu.readMemoryPtr(sp + (4*emu.psize))    # in: ObjectAttributes
        arg4 = emu.readMemoryPtr(sp + (5*emu.psize))    # out: IoStatusBlock
        arg5 = emu.readMemoryPtr(sp + (6*emu.psize))    # in: ShareAccess
        arg6 = emu.readMemoryPtr(sp + (7*emu.psize))    # in: OpenOptions

        print("ntOpenFile( 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x )" % (arg1, arg2, arg3, arg4, arg5, arg6))
        length, rootdir, objname, attrib, secdesc, secqos = \
                emu.readMemoryFormat(arg3, "<IPPIPP")
        print("  OBJECT_ATTRIBUTES: %r %r %r %r %r %r" % (length, rootdir, objname, attrib, secdesc, secqos))
        import envi.interactive as ei; ei.dbg_interact(locals(), globals())


        # for now
        retval = 0
        emu.setRegister(0, retval)

    def sys_win_NtCreateSection(self, emu, op):
        stackDump(emu)
        ### NOT DONE
        sp = emu.getStackCounter()
        arg0 = emu.readMemoryPtr(sp + emu.psize)
        arg1 = emu.readMemoryPtr(sp + (2*emu.psize))
        arg2 = emu.readMemoryPtr(sp + (3*emu.psize))
        arg3 = emu.readMemoryPtr(sp + (4*emu.psize))
        arg4 = emu.readMemoryPtr(sp + (5*emu.psize))
        arg5 = emu.readMemoryPtr(sp + (6*emu.psize))

        print("ntCreateSection( 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x )" % (arg0, arg1, arg2, arg3, arg4, arg5))
        import envi.interactive as ei; ei.dbg_interact(locals(), globals())

        # for now
        retval = 0
        emu.setRegister(0, retval)

    def sys_win_MapViewOfSection(self, emu, op):
        stackDump(emu)
        ### NOT DONE
        sp = emu.getStackCounter()
        arg0 = emu.readMemoryPtr(sp + emu.psize)
        arg1 = emu.readMemoryPtr(sp + (2*emu.psize))
        arg2 = emu.readMemoryPtr(sp + (3*emu.psize))
        arg3 = emu.readMemoryPtr(sp + (4*emu.psize))
        arg4 = emu.readMemoryPtr(sp + (5*emu.psize))
        arg5 = emu.readMemoryPtr(sp + (6*emu.psize))

        print("ntMapViewOfSection( 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x )" % (arg0, arg1, arg2, arg3, arg4, arg5))
        import envi.interactive as ei; ei.dbg_interact(locals(), globals())

        # for now
        retval = 0
        emu.setRegister(0, retval)


class LinuxKernel(dict):
    def __init__(self, emu):
        dict.__init__(self)
        self.emu = emu

        
        # setup key files db here
        self['fs'] = collections.defaultdict(dict)    # perhaps create file objects, for now this.
        self['fhandles'] = {}   # store a connection between a handle and a member of 'fs'

        # actual syscall handlers
        win_syscalls = {
            }


    def op_sysenter(krnl, emu, op):
        # handle select Windows syscalls
        callnum = emu.getRegister(0)
        syscall = krnl.win_syscalls.get(callnum)
        if syscall is not None:
            syscall(emu, op)

        else:
            raise SystemCallNotImplemented(callnum, emu, op)




def backTrace(emu):
    '''
    Work through the emulator stack looking for return pointers
    '''
    sp = emu.getStackCounter()
    stackmap = emu.getMemoryMap(sp)
    stacktop = stackmap[0] + stackmap[1]
    while sp < stacktop:
        #print("[D] 0x%x < 0x%x" % (sp, stacktop))
        cur = emu.readMemoryPtr(sp)
        curmap = emu.getMemoryMap(cur)
        if curmap:
            cmmva, cmmsz, cmmperms, cmmname = curmap
            tmpva = max(cur-7, curmap[0])
            tmpsz = cur - tmpva
            prevmem = emu.readMemory(tmpva, tmpsz)

            while tmpva < cur:
                try:
                    op = emu.parseOpcode(tmpva)
                    #print("0x%x: %r" % (op.va, op))
                except:
                    tmpva += 1
                    continue

                if tmpva + len(op) == cur and op.isCall():
                    # this looks like a good call in our call stack
                    tgtfname = 'None'
                    if self.vw:
                        funcname = self.vw.getName(self.vw.getFunction(op.va))
                        tgtvas = [bva for bva, bflags in op.getBranches(emu=emu) if not bflags & envi.BR_FALL]
                        if len(tgtvas) and self.vw:
                            tgtva = tgtvas[0]
                            tgtfname = self.vw.getName(tgtva)

                    print("%r   %r   0x%x -> %r" % (cmmname, funcname, op.va, tgtfname))
                tmpva += 1
    
        sp += emu.psize


def stackDump(emu, count=16):
    '''
    Dump Stack, including derefs
    '''
    # TODO: recurse through pointers
    # TODO: list registers that point at any of the pointers/stackaddrs
    print("Stack Dump:")
    sp = emu.getStackCounter()
    for x in range(count):
        val = emu.readMemoryPtr(sp)
        valmap = emu.getMemoryMap(val)
        if valmap and emu.vw:
            bytesleft = (valmap[0] + valmap[1]) - val
            if bytesleft >= emu.psize:
                valptr = emu.readMemoryPtr(val)
                if emu.getMemoryMap(valptr):    # isValidPointer for emus
                    strdata = hex(valptr)
                else:
                    strdata = repr(emu.readMemory(val, min(24, bytesleft)))
            else:
                strdata = repr(emu.readMemory(val, bytesleft))

            print("\t0x%x:\t0x%x \t-> %s" % (sp, val, strdata))
        else:
            print("\t0x%x:\t0x%x" % (sp, val))
        sp += emu.psize

def heapDump(emu):
    '''
    Dump the Heap allocations
    '''
    print("Stack Dump:")
    heap = getHeap(emu)
    print(heap.dump())

class TestEmulator:
    def __init__(self, emu, vw=None, verbose=False, fakePEB=False, guiFuncGraphName=None, hookfuncsbyname=False):
        '''
        Instiate a TestEmulator harness.  This holds and controls an emulator object.

        emu -       an existing emulator
        vw -        VivWorkspace object to build an emulator from
        verbose -   print log messages
        fakePEB -   set up fake PEB/TEB memoryspaces and setup the appropriate segment
        guiFuncGraphName - name of the gui window to send location info to (nav info)
        hookbyname - should we 
        '''
        self.vw = None
        self.vwg = None

        self.emu = emu
        self.emu.temu = self    # i know i'm evil.

        if vw is not None:
            self.vw = emu.vw = vw
            self.vwg = self.vw.getVivGui()
        elif hasattr(emu, 'vw'):
            self.vw = emu.vw
            self.vwg = self.vw.getVivGui()

        self.verbose = verbose
        self.guiFuncGraphName = guiFuncGraphName

        self.XWsnapshot = {}
        self.cached_mem_locs = []
        self.call_handlers = {}
        self.environment = {}
        self.fs = {}

        self.hookFuncs(importonly = not hookfuncsbyname)

        self.teb = None
        self.peb = None
        if fakePEB:
            self.initFakePEB()

    def setFileInfo(self, filename, filebytes, fileattrib=0):
        '''
        Add/Replace Filesystem data
        filedict is a dictionary of attributes/timestamps
        '''
        self.fs[filename] = (fileattrib, filebytes)

    def initFakePEB(self):
        '''
        This is currently i386 only
        '''
        peb = self.emu.findFreeMemoryBlock(PEBSZ, 0x7ffd3000)
        self.emu.addMemoryMap(peb, 6, 'FakePEB', b'\0'*PEBSZ)
        teb = self.emu.findFreeMemoryBlock(TEBSZ, 0x7ffdc000)
        self.emu.addMemoryMap(teb, 6, 'FakeTEB', b'\0'*TEBSZ)

        self.emu.writeMemoryPtr(teb+0x30, peb)
        # fake TEB: c4eea6060000a70600e0a60600000000001e0000000000000040fd7f00000000401700000c140000000000002c40fd7f00c0fd7f000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000090400000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000

        # fake PEB: 00000108ffffffff0000e7008088d277b817400000000000000040008083d27700000000000000000100000038d5dc7700000000000000000000eb77000000006082d277ffffffff0700000000006f7f0000000090056f7f0000fb7f2402fc7f4806fd7f01000000000000000000000000809b076de8ffff000010000020000000000100001000000c000000100000000085d27700005d0000000000140000004083d2770600000001000000b11d00010200000003000000060000000000000001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000

        if self.emu.psize == 4:
            self.emu.setSegmentInfo(e_i386.SEG_FS, teb, TEBSZ)
        else:
            self.emu.setSegmentInfo(e_i386.SEG_GS, teb, TEBSZ)


    def hookFuncs(self, importonly=True):
        if not hasattr(self.emu, 'vw') or self.emu.vw is None:
            return

        for impva, impsz, imptype, impname in self.emu.vw.getImports():
            fname, funcname = impname.split('.', 1)
            if impname in import_map:
                self.call_handlers[impva] = import_map.get(impname)
            elif "*." + funcname in import_map:
                impname = "*." + funcname
                self.call_handlers[impva] = import_map.get(impname)

        if importonly:
            return

        for va, name in self.emu.vw.getNames():
            #print(name, '%.8x' % va)
            if name.endswith("%.8x" % va):
                name = name[:-9]
                #print("checking %r" % name)


            if name in import_map:
                self.call_handlers[va] = import_map.get(name)
                print("Mapping call_handler by *name*: %r => 0x%x" % (name, va))

    def printMemStatus(self, op=None, use_cached=False):
        emu = self.emu
        pc = emu.getProgramCounter()
        SP = emu.getStackCounter()
        if op is None:
            return

        done = []
        if use_cached:
            addrs = self.cached_mem_locs
        else:
            addrs = []
            for oper in op.opers:
                try:
                    # value
                    addr = oper.getOperValue(op, emu)
                    if type(addr) == int:
                        if addr not in addrs:
                            addrs.append(addr)
                    # address
                    if oper.isDeref():
                        addr = oper.getOperAddr(op, emu)
                        if addr is not None:
                            if addr not in addrs:
                                addrs.append(addr)

                except Exception as e:
                    print("error: %s" % e)

        for addr in addrs:
            if not emu.isValidPointer(addr):
                #if emu.vw.verbose:
                #    if type(addr) == int:
                #        print("No valid memory at address: 0x%x" % addr)
                #    else:
                #        print("No valid memory at address: %s" % addr)
                continue

            print(self.XW(addr, snapshot=SNAP_SWAP))
        self.cached_mem_locs = addrs


    def XW(self, address, length = 32, dwperline = 8, snapshot=0):
        output = []
        mm = self.emu.getMemoryMap(address)
        if mm is None:
            return ''

        mmva, mmsz, mmperm, mmname = mm
        if mmva+mmsz < address + (length*4):
            goodbcnt = (mmva+mmsz-address)
            diff = (length*4) - goodbcnt
            bs = self.emu.readMemory(address, goodbcnt)
            bs += b'A' * diff

        else:
            bs = self.emu.readMemory(address, length*4)

        for i in range(length):
            addr = address + (i * 4)
            if (i % dwperline == 0):
                output.append("%.08x:\t "%(addr))

            data = bs[i*4:(i*4)+4]

            # do the snapshotting thing
            pre = post = ''
            if snapshot in (SNAP_DIFF, SNAP_SWAP):
                sdata = self.XWsnapshot.get(addr) 
                if sdata is not None and sdata != data:
                    # highlight the text somehow
                    pre = '\x1b[7m'
                    post = '\x1b[27m'

            if snapshot in (SNAP_CAP, SNAP_SWAP):
                self.XWsnapshot[addr] = data
            output.append(pre + data.hex() + post)

            if ((i+1) % dwperline == 0):
                output.append("\n")
            else:
                output.append("  ")

        return "".join(output)


    def showPriRegisters(self, snapshot=SNAP_NORM):
        emu = self.emu
        print("\nRegisters:")
        reggrps = emu.vw.arch.archGetRegisterGroups()
        for name, gen_regs in reggrps:
            if name == 'general':
                break

        reg_table, meta_regs, PC_idx, SP_idx, reg_vals = emu.getRegisterInfo()
        if isinstance(emu, vtrace.Trace):
            reg_table, meta_regs, PC_idx, SP_idx, reg_vals = emu.getRegisterContext().getRegisterInfo()
        reg_dict = { reg_table[i][0] : (reg_table[i][1], reg_vals[i]) for i in range(len(reg_table)) }

        # print(through the various registers)
        for i in range(len(gen_regs)):
            rname = gen_regs[i]
            rsz, rval = reg_dict.get(rname)

            # line break every so often
            if (i%5 == 0):
                sys.stdout.write("\n")#%4x"%i)

            # do snapshotting:
            pre = post = ''
            if snapshot in (SNAP_DIFF, SNAP_SWAP):
                srval = self.XWsnapshot.get(rname) 
                if srval is not None and srval != rval:
                    # highlight the text somehow
                    pre = '\x1b[7m'
                    post = '\x1b[27m'

            if snapshot in (SNAP_CAP, SNAP_SWAP):
                self.XWsnapshot[rname] = rval

            rnpad = ' ' * (11 - len(rname))

            fmt = "%%s%%s: %%%dx%%s" % (rsz//4)
            sys.stdout.write(fmt % (rnpad, pre + rname, rval, post))

        # Line feed
        print("\n")

    def showFlags(self):
        """
        Show the contents of the Status Register
        """
        #print("\tStatus Flags: \tRegister: %s\n" % (bin(self.getStatusRegister())))
        try:
            print("\tStatFlags: " + '\t'.join(["%s %s" % (f,v) for f,v in self.emu.getStatusFlags().items()]))
        except Exception as e:
            print("no flags: ", e)


    def backTrace(self):
        emu = self.emu
        backTrace(emu)

    def stackDump(self, count=16):
        # TODO: recurse through pointers
        # TODO: list registers that point at any of the pointers/stackaddrs
        emu = self.emu
        stackDump(emu, count)

    def heapDump(self):
        emu = self.emu
        heapDump(emu)

    def printStats(self, i):
        curtime = time.time()
        dtime = curtime - self.startRun
        print("since start: %d instructions in %.3f secs: %3f ops/sec" % \
                (i, dtime, i//dtime))

    def runStep(self, maxstep=1000000, follow=True, showafter=True, runTil=None, pause=True, silent=False, finish=0, tracedict=None):
        '''
        runStep is the core "debugging" functionality for this emulation-helper.  it's goal is to 
        provide a single-step interface somewhat like what you might get from a GDB experience.  

        pertinent registers are printed with their values, the current instruction, and any helpers
        that the operands may point to in memory (as appropriate).

        special features:
        [ function arguments ]
        * tracedict allows code to be evaluated and printed at specific addresses: 
                tracedict={va:'python code here', 'locals':{'something':4}}

        * call_handlers dict (global in the library) allows swapping in our python code in place of 
            calls to other binary code, like memcpy, or other code which may fail in an emulator

        * follow - should the emulator follow calls?

        * showafter - show memory and operands *after* emulating the instruction

        * runTil - duplication of "finish" - will be removed in the future
        
        * pause - do we stop at each instruction?
        
        * silent - do we print out status after each instruction?
        
        * finish - when the Program Counter reaches this address, stop
        
        [ interactive cli ]
        * cli interface allows viewing and modifying memory/python objects:
                rax
                [rax]
                [rax:23]
                [rax+8:4]
                [0xf00b4:8]
                rax=42
                [0xf00b4]=0x47145
        * cli allows skipping printing  (arg:silent=True)
                silent=True
        * cli allows running until a VA without pauses: 
                go 0x12345
        * cli allows executing until next branch:
                b
        * cli allows dumping the stack:
                stack
        * cli allows viewing/setting the Program Counter:
                pc
                pc=0x43243
        * cli allows skipping instructions:
                skip
        * cli allows numerous libc-style functions:
                memset
                memcpy
                strcpy
                strncpy
                strcat
                strlen
        
        '''
        emu = self.emu
        self._follow = follow
        self.op_handlers = {}   # for instructions like 'sysenter' which are not supported by the emu

        plat = emu.vw.getMeta('Platform')
        if plat.startswith('win'):
            self.kernel = WinKernel(emu)
        #elif plat.startswith('linux') or plat:
        else:   # FIXME: need to make Elf identification better!!
            self.kernel = LinuxKernel(emu)

        self.op_handlers['sysenter'] = self.kernel.op_sysenter

        mcanv = e_memcanvas.StringMemoryCanvas(emu, syms=emu.vw)
        self.mcanv = mcanv  # store it for later inspection

        # set up tracedict
        if tracedict is None:
            tracedict = {}
        else:
            print("tracedict entries for %r" % (','.join([hex(key) for key in tracedict.keys() if type(key) == int])))


        nonstop = 0
        tova = None
        quit = False
        moveon = False
        emuBranch = False
        silentUntil = None

        # set silentExcept to include all tracedict items
        silentExcept = [va for va, expr in tracedict.items() if expr is None]

        i = 0
        self.startRun = time.time()
        while maxstep > i:
            try:
                skip = skipop = False
                i += 1

                pc = emu.getProgramCounter()
                if pc in (runTil, finish):
                    print("PC reached 0x%x." % pc)
                    break

                op = emu.parseOpcode(pc)
                self.op = op    # store it for later in case of post-mortem

                # cancel emuBranch as we've come to one
                if op.isReturn() or op.isCall():
                    emuBranch = False

                #### TRACING 
                for key in (pc, 'ALL'):
                    tdata = tracedict.get(key)
                    if tdata is not None:
                        try:
                            lcls = locals()
                            outlcls = tracedict.get('locals')
                            if outlcls is not None:
                                lcls.update(outlcls)

                            lcls.update(emu.getRegisters())
                            if isinstance(emu, vtrace.Trace):
                                lcls.update(emu.getRegisterContext().getRegisters())

                            print(repr(eval(tdata, globals(), lcls)))
                        except Exception as e:
                            print("TraceMonitor ERROR at 0x%x: %r" % (pc, e))

                ####

                if silentUntil == pc:
                    silent = False
                    silentUntil = None
                    self.printStats(i)

                if silent and not pc in silentExcept:
                    showafter = False
                else:
                    # do all the interface stuff here:
                    self.showPriRegisters(snapshot=SNAP_SWAP)
                    self.showFlags() # ARM fails this right now.
                    try:
                        self.printMemStatus(op)
                    except Exception as e:
                        print("MEM ERROR: %s:    0x%x %s" % (e, op.va, op))
                        import traceback
                        traceback.print_exc()

                    print("Step: %s" % i)
                    mcanv.clearCanvas()
                    try:
                        op.render(mcanv)
                    except Exception as e:
                        print("ERROR rendering opcode: %r" % e)

                    extra = self.getNameRefs(op)

                    opbytes = emu.readMemory(pc,len(op))
                    print("%.4x\t%20s\t%s\t%s"%(pc,hexlify(opbytes),mcanv.strval, extra))

                    print("---------")
                    prompt = "q<enter> - exit, eval code to execute, 'skip' an instruction, 'b'ranch, 'go [+]#' to va or +# instrs or enter to continue: "

                    # nonstop controls whether we stop.  tova indicates we're hunting for a va, otherwise 
                    # treat nonstop as a negative-counter
                    if tova is not None:
                        if pc == tova:
                            nonstop = 0

                    elif nonstop:
                        nonstop -= 1

                    if not (emuBranch or nonstop) and pause:
                        tova = None
                        nextva = op.va + len(op)
                        moveon = False

                        # send the selected GUI window to current program counter
                        if self.guiFuncGraphName is not None and not silent:
                            if self.vwg is not None:
                                self.vwg.sendFuncGraphTo(pc, self.guiFuncGraphName)
                            else:
                                print("can't send FuncGraph to 0x%x because we don't have a handle to the Viv GUI" % pc)

                        # UI Interface!  interact with the user
                        uinp = input(prompt)
                        while len(uinp) and not (moveon or quit or emuBranch):
                            try:
                                if uinp == "q":
                                    quit = True
                                    break

                                elif uinp.startswith('silent'):
                                    parts = uinp.split(' ')
                                    silentUntil = parseExpression(emu, parts[-1], {'next':nextva})
                                    print("silent until 0x%x" % silentUntil)
                                    silent = True

                                elif uinp in ('backtrace', 'bt'):
                                    self.backTrace()
                                    moveon = True
                                    break

                                elif uinp.startswith('go '):
                                    args = uinp.split(' ')

                                    if args[-1].startswith('+'):
                                        nonstop = parseExpression(emu, args[-1], {'next': nextva})
                                    else:
                                        tova = parseExpression(emu, args[-1], {'next': nextva})
                                        nonstop = 1
                                    break

                                elif uinp == 'ni':
                                    # next instruction (eg. skip over a call)
                                    nonstop = 1
                                    tova = nextva
                                    break

                                elif uinp in ('b', 'branch'):
                                    emuBranch = True
                                    break

                                elif uinp.startswith('stack'):
                                    count=16
                                    if ' ' in uinp:
                                        cmd, ctstr = uinp.split(' ', 1)
                                        try:
                                            count = int(ctstr, 0)
                                        except ValueError as e:
                                            print(e)

                                    self.stackDump(count)
                                    moveon = True
                                    break

                                elif uinp == 'heap':
                                    self.heapDump()
                                    moveon = True
                                    break

                                elif uinp == 'refresh':
                                    # basically does a NOP, doesn't change anything, just let the data be reprinted.
                                    moveon = True
                                    break

                                elif uinp.startswith('pc=') or uinp.startswith('pc ='):
                                    print("handling setProgramCounter()")
                                    args = uinp.split('=')
                                    newpc = parseExpression(emu, args[-1])
                                    print("new PC: 0x%x" % newpc)
                                    emu.setProgramCounter(newpc)
                                    moveon = True
                                    break

                                elif '=' in uinp:
                                    print("handling generic register/memory writes")
                                    args = uinp.split('=')
                                    data = args[-1].strip() #   .split(',')  ??? why did i ever do this?

                                    if '[' in args[0]:
                                        # memory derefs
                                        tgt = args[0].replace('[','').replace(']','').split(':')
                                        addrstr = tgt[0]
                                        memaddr = parseExpression(emu, addrstr)

                                        if len(tgt) > 1:
                                            if tgt[-1] not in ('h', 'H', 's', 'S'):
                                                size = parseExpression(emu, tgt[-1])
                                        else:
                                            size = emu.psize


                                        if data.startswith('"') and data.endswith('"'):
                                            # write string data
                                            bdata = (data[1:-1]).encode()
                                            emu.writeMemory(memaddr, bdata)
                                        elif tgt[-1] in ('h', 'H'):
                                            bdata = bytes.fromhex(data.encode())
                                            emu.writeMemory(memaddr, bdata)
                                        else:
                                            # write number
                                            emu.writeMemValue(memaddr, parseExpression(emu, data), size)

                                    else:
                                        # must be registers
                                        emu.setRegisterByName(args[0], parseExpression(emu, data))

                                elif uinp.strip().startswith('[') and ']' in uinp:
                                    try:
                                        idx = uinp.find('[') + 1
                                        eidx = uinp.find(']', idx)
                                        expr = uinp[idx:eidx]
                                        print("handling memory read at [%s]" % expr)
                                        size = emu.getPointerSize()
                                        if ':' in expr:
                                            nexpr, nsize = expr.rsplit(':',1)
                                            va = parseExpression(emu, nexpr)
                                            if nsize in ('s', 'S'):
                                                data = emu.readMemString(va)
                                                print("[%s] == %r" % (expr, data))
                                            elif nsize in ('w', 'W'):
                                                data = readMemString(emu, va, wide=True)
                                                print("[%s] == %r" % (expr, data))
                                            elif nsize in ('u', 'U'):
                                                data = readMemString(emu, va, wide=True)
                                                print("[%s] == %r" % (expr, data.decode('utf-16le')))
                                            else:
                                                try:
                                                    size = parseExpression(emu, nsize)
                                                    data = emu.readMemory(va, size)
                                                    print("[%s:%s] == %r" % (nexpr, size, data.hex()))
                                                except Exception as e:
                                                    # if number fails, just continue with a default size and the original expr
                                                    print("unknown size: %r.  using default size." % size)

                                        else:
                                            va = parseExpression(emu, expr)
                                            data = emu.readMemory(va, size)
                                            print("[%s:%s] == %r" % (expr, size, data.hex()))
                                    except Exception as e:
                                        print("ERROR: %r" % e)

                                elif uinp == 'skip':
                                    newpc = emu.getProgramCounter() + len(op)
                                    print("new PC: 0x%x" % newpc)
                                    skipop = True
                                    break

                                # the following functions are DEPRECATED
                                elif uinp == 'memset':
                                    print(memset(emu))
                                    skipop = True
                                elif uinp == 'memcpy':
                                    print(memcpy(emu))
                                    skipop = True
                                elif uinp == 'strcpy':
                                    print(strcpy(emu))
                                    skipop = True
                                elif uinp == 'strncpy':
                                    print(strncpy(emu))
                                    skipop = True
                                elif uinp == 'strcat':
                                    print(strcat(emu))
                                    skipop = True
                                elif uinp == 'strlen':
                                    print(strlen(emu))
                                    skipop = True

                                elif uinp == 'malloc':
                                    print(malloc(emu))
                                    skipop = True
                                else:
                                    try:
                                        lcls = {'next': nextva}
                                        lcls.update(locals())
                                        lcls.update(emu.getRegisters())
                                        if isinstance(emu, vtrace.Trace):
                                            lcls.update(emu.getRegisterContext().getRegisters())

                                        out = eval(uinp, globals(), lcls)

                                        taint = emu.getVivTaint(out)
                                        if taint:
                                            out = "taint: %s: %s" % (taint[1], emu.reprVivTaint(taint))
                                        
                                        if type(out) == int:
                                            print(hex(out))
                                        else:
                                            print(out)
                                    except:
                                        import sys
                                        sys.excepthook(*sys.exc_info())

                            except:
                                import traceback
                                traceback.print_exc()

                            #self.printStats(i)
                            uinp = input(prompt)

                if quit:
                    print("Quitting!")
                    self.printStats(i)
                    return

                if moveon:
                    continue
                '''
                # handle Calls separately
                if op.isCall() and not skipop:
                    self.dbgprint("Call...")
                    handler = None
                    for brva, brflags in op.getBranches(emu=emu):
                        if brflags & envi.BR_FALL:
                            continue

                        taint = emu.getVivTaint(brva)
                        if taint:
                            taintval, tainttype, tainttuple = taint
                            brva = tainttuple[0]

                        self.dbgprint("brva: 0x%x  brflags: 0x%x" % (brva, brflags))
                        handler = self.call_handlers.get(brva)
                        if handler is not None:
                            break

                    self.dbgprint( " handler for call to (0x%x): %r" % (brva, handler))
                    if handler is not None:
                        handler(emu, op)
                        skipop = True

                    elif self._follow and not skip and not skipop:
                        # use the emulator to execute the call
                        starteip = emu.getProgramCounter()
                        if hasattr(emu, 'emumon') and emu.emumon is not None:
                            emu.emumon.prehook(emu, op, starteip)

                        emu.executeOpcode(op)
                        endeip = emu.getProgramCounter()
                        i += 1
                        if hasattr(emu, 'emumon') and emu.emumon is not None:
                            emu.emumon.posthook(emu, op, endeip)

                        self.dbgprint("starteip: 0x%x, endeip: 0x%x  -> %s" % (starteip, endeip, emu.vw.getName(endeip)))
                        if hasattr(emu, 'curpath'):
                            vg_path.getNodeProp(emu.curpath, 'valist').append(starteip)
                        skip = True
                '''
                if op.isCall() or op.iflags & envi.IF_BRANCH:
                    skip, skipop = self.handleBranch(op, skip, skipop)

                # if not already emulated a call, execute the instruction here...
                if not skip and not skipop:
                    # execute opcode.  if unsupported, look for op_handlers
                    failed = False
                    try:
                        emu.stepi()
                    except e_exc.UnsupportedInstruction:
                        failed = True

                    # check for failure, and look for an op_handler. then raise an exception
                    if failed:
                        ophndlr = self.op_handlers.get(op.mnem)

                        if ophndlr is not None:
                            print("opcode handler: %r" % ophndlr)
                            newpc = ophndlr(emu, op)
                            if not newpc:
                                newpc = op.va + len(op)

                            emu.setProgramCounter(newpc)

                        else:
                            import sys
                            sys.excepthook(*sys.exc_info())
                            break


                    # print the updated latest stuff....
                    if showafter:
                        try:
                            extra = self.getNameRefs(op)
                            if len(extra):
                                print("after:\t%s\t%s"%(mcanv.strval, extra))

                            self.printMemStatus(op, use_cached=True)
                        except Exception as e:
                            print("MEM ERROR: %s:    0x%x %s" % (e, op.va, op))
                            import sys
                            sys.excepthook(*sys.exc_info())

                # unless we've asked to skip the instruction...
                elif skipop:
                    newpc = emu.getProgramCounter() + len(op)
                    emu.setProgramCounter(newpc)

            except KeyboardInterrupt:
                self.printStats(i)
                break

            except envi.SegmentationViolation:
                pc = emu.getProgramCounter()
                taint = emu.getVivTaint(pc)
                if op.isCall() or op.iflags & envi.IF_BRANCH and taint:
                    skip, skipop = self.handleBranch(op, skip, skipop)
                else:
                    import sys
                    sys.excepthook(*sys.exc_info())
                    break

            except:
                import sys
                sys.excepthook(*sys.exc_info())
                nonstop = 0

        self.printStats(i)

    def handleBranch(self, op, skip, skipop):
        '''
        '''
        emu = self.emu
        handler = None
        for brva, brflags in op.getBranches(emu=emu):
            if brflags & envi.BR_FALL:
                continue

            if hasattr(emu, 'getVivTaint'):
                taint = emu.getVivTaint(brva)
                if taint:
                    taintval, tainttype, tainttuple = taint
                    brva = tainttuple[0]

            self.dbgprint("brva: 0x%x  brflags: 0x%x" % (brva, brflags))
            handler = self.call_handlers.get(brva)
            if handler is not None:
                break

        self.dbgprint( " handler for call to (0x%x): %r" % (brva, handler))
        if handler is not None:
            if op.isCall():
                # apply return address. any supported arch callconv should
                # get this right... so take the first one.
                retva = op.va + len(op)
                ccname, cconv = emu.getCallingConventions()[0]
                cconv.allocateReturnAddress(emu)    # this assumes we've called
                cconv.setReturnAddress(emu, retva)
                print("setting return address to 0x%x" % retva)

            handler(emu, op)
            skip = True
            if not op.isCall():
                # this was a branch... our handlers are intended to handle calls.
                print("handleBranch(): have handler, but %r (at 0x%x) is not a call, may work fine, but may not." % (op, op.va))


        elif self._follow and not skip and not skipop:
            # use the emulator to execute the call
            starteip = emu.getProgramCounter()
            if hasattr(emu, 'emumon') and emu.emumon is not None:
                emu.emumon.prehook(emu, op, starteip)

            emu.executeOpcode(op)
            endeip = emu.getProgramCounter()
            if hasattr(emu, 'emumon') and emu.emumon is not None:
                emu.emumon.posthook(emu, op, endeip)

            self.dbgprint("starteip: 0x%x, endeip: 0x%x  -> %s" % (starteip, endeip, emu.vw.getName(endeip)))
            if hasattr(emu, 'curpath'):
                vg_path.getNodeProp(emu.curpath, 'valist').append(starteip)
            skip = True
        return skip, skipop

    def dbgprint(self, *args, **kwargs):
        if self.verbose:
            data = '\t'.join(args)
            print(data)

    def getNameRefs(self, op):
        emu = self.emu
        extra = ''
        ###  HACK: NOT FOR PUBLIC CONSUMPTION:
        #taintPause = emu._pause_on_taint
        #emu._pause_on_taint = False
        try:

            for operidx, oper in enumerate(op.opers):
                opval = oper.getOperValue(op, emu)
                if type(opval) == int:
                    opnm = emu.vw.getName(opval)
                    if opnm is None and hasattr(emu, 'getVivTaint'):
                        taint = emu.getVivTaint(opval)
                        if taint:
                            taintrepr = emu.reprVivTaint(taint)
                            opnm = "%s (%s)" % (taint[1], taintrepr)

                    if opnm is not None:
                        extra += '\t; $%d = %r' % (operidx, opnm)

                if oper.isDeref():
                    dopval = oper.getOperAddr(op, emu)
                    if type(dopval) == int:
                        dopnm = emu.vw.getName(dopval)
                        if opnm is None and hasattr(emu, 'getVivTaint'):
                            taint = emu.getVivTaint(opval)
                            if taint:
                                taintrepr = emu.reprVivTaint(taint)
                                opnm = "%s (%s)" % (taint[1], taintrepr)

                        if dopnm is not None:
                            extra += '\t; &$%d = %r' % (operidx, dopnm)

        except Exception as e:
            print("getNameRefs: ERROR: %r" % e)
        #finally:
        #    emu._pause_on_taint = taintPause
        return extra

    def runUntil(self, eip=0, mnem=None, maxstep=1000000):
        emu = self.emu
        for i in range(maxstep):
            pc = emu.getProgramCounter()
            op = emu.parseOpcode(pc)
            opbytes = emu.readMemory(pc,len(op))
            if pc == eip or op.mnem == mnem:
                break
            emu.stepi()
        runStep(emu)

    def printWriteLog(self):
        print('\n'.join(['0x%.8x: 0x%.8x << %32r %r' % (x,y,d.hex(),d) for x,y,d in self.emu.path[2].get('writelog')]))


    def insertReadWriteComments(self, vw):
        for va, tva, data in self.emu.path[2].get('readlog'):
            insertComment(vw, va, "[r:%x] %r (%r)" % (tva, data.hex(), data))

        for va, tva, data in self.emu.path[2].get('writelog'):
            insertComment(vw, va, "[W:%x] %r (%r)" % (tva, data.hex(), data))

from envi.memory import MM_READ
def readMemString(self, va, maxlen=0xfffffff, wide=False):
    '''
    Returns a C-style string from memory.  Stops at Memory Map boundaries, or the first NULL (\x00) byte.
    '''

    if wide:
        term = b'\0\0'
    else:
        term = b'\0'

    for mva, mmaxva, mmap, mbytes in self._map_defs:
        if mva <= va < mmaxva:
            mva, msize, mperms, mfname = mmap
            if not mperms & MM_READ:
                raise envi.SegmentationViolation(va)
            offset = va - mva

            # now find the end of the string based on either \x00, maxlen, or end of map
            end = mbytes.find(term, offset)

            left = end - offset
            if end == -1:
                # couldn't find the NULL byte
                mend = offset + maxlen
                cstr = mbytes[offset:mend]
            else:
                # couldn't find the NULL byte go to the end of the map or maxlen
                if wide and (left & 1):
                    left += 1
                mend = offset + (maxlen, left)[left < maxlen]
                cstr = mbytes[offset:mend]
            return cstr

    raise envi.SegmentationViolation(va)



import vivisect.impemu.monitor as vi_mon
import vivisect.symboliks.analysis as vs_anal

class SymbolikTraceAnalMod(vi_mon.AnalysisMonitor):
    def __init__(self, emu):
        self.emu = emu
        self.sctx = vs_anal.getSymbolikAnalysContext(emu.vw)
        self.xlate = self.sctx.getTranslator()

    def prehook(self, emu, op, starteip):
        self.xlate.translateOp(op)

def insertComment(vw, va, comment):
    curcmt = vw.getComment(va)
    if curcmt is not None:
        vw.setComment(va, "%s  ; %s" % (comment, curcmt))
    else:
        vw.setComment(va, comment)



if __name__ == "__main__":
    main(sys.argv)




import sys

TEXTMIN = '\x20'
TEXTMAX = '\x7e'
IGNOREBIN = '\x09\x0a\x0d\x00'

def checkDATA(bytes, COEFF = 4):
        text = 0
        binary = 0
        for c in bytes:
                if (c >= TEXTMIN and c <= TEXTMAX):
                        text += 1
                else:
                        if (not c in IGNOREBIN):
                            binary += 1
        if ((text) > (COEFF * binary)):
                return True
        return False

def hexText(bytes):
        """
        hexText accepts a string and returns a HEX-ified version of it.
        """
        out = ""
        for ch in bytes:
                out += "\\x%.02x"%(ord(ch))
        return out

def SmartOutput(string, TEXTCOEFF=4):
        if (not checkDATA(string, TEXTCOEFF)):
                string = hexText(string)
        return string

def SmartPrint(string, COEFF=4):
	print(SmartOutput(string, COEFF))


def dumpTree(subtree, title="TREE:", outf = sys.stdout, ignore=(), deep=False, cb=None):
    """ 
    dumpTree() is a recursive sub which walks a tree-like data structure and ouputs the findings to the file of your choice
    stdout is the default output file, however, it could easily be used thus:
        dumpTree(mystrucureobject, "WEIRDO TREE", outf = file("dumptree.out","w"))
    """
    if (type(subtree) == list or type(subtree) == tuple):
        print >>outf,("%s: (%s)"%(title,type(subtree)))
        for item in range(len(subtree)):
            dumpTree(subtree[item], "%s:[%d]"%(title,item), outf, ignore)

    elif (type(subtree) == dict):
        print >>outf,("%s: (%s)"%(title,type(subtree)))
        for i in (subtree).keys():
            istr = repr(i)
            if istr in ignore:
                continue
            item = (subtree).get(i)
            dumpTree(item, "%s:%s"%(title,istr), outf, ignore)

    elif (repr(type(subtree)) == "<type 'instance'>"):
        print >>outf,("%s: (%s)"%(title,type(subtree)))
        if cb:
            cb(title, subtree)

        stvars = vars(subtree)
        for i in stvars.keys():
            istr = repr(i)
            if i in ignore:
                continue
            item = stvars.get(i)
            dumpTree(item, "%s:%s"%(title, istr), outf, ignore)

        if deep:
            for item in dir(subtree):
                istr = repr(item)
                if item in ignore:
                    continue
                dumpTree(item, "%s:%s"%(title, istr), outf, ignore)

    else:
        if (type(subtree) == str and len(subtree)>40):
            print >>outf,("%s %s == %s ..... (size = %d bytes)"%(title, type(subtree), subtree[:40], len(subtree)))
        elif (type(subtree) == int):
            print >>outf,("%s %s == 0x%x "%(title, type(subtree), subtree))
        else:
            print >>outf,("%s %s == %s "%(title, type(subtree), subtree))

"""
>>> for i in \
... vars(pe).keys():
...   print i
...   type(vars(pe).get(i))

"""



import math
import time
import random
import vivisect
import threading

from visual import *

# rate constants
STABILITY_CONST =   2
VELOCITY_COEFF =    .05     # dampen the effects a little
SPEED_LIMIT =       10

# force constants
#GRAVITY_COEFF =     .049
GRAVITY_COEFF =     .02
CENTER_PULL =       .05
FRICTION_COEFF =    .1

# log force constants  (these forces vary by distance)
NODE_FORCE_COEFF = .01
EDGE_FORCE_COEFF = .15


class Node(sphere):
    def __init__(self, system, ident, color=color.blue):
        x = random.randint(1, system.ceiling)
        y = system.ceiling
        z = random.randint(1, system.ceiling)
        pos = vector(x,y,z)
        sphere.__init__(self, pos=pos, radius=1)
        self.system = system
        self.edges = []
        self.ident = ident
        self.color = color

        self.velocity = vector(0,0,0)
        self.impulse = vector(0,0,0)

    def __sub__(self, other):
        return other.pos - self.pos 

    def isCollision(self, other):
        distance = self - other
        mindistance =  self.radius+other.radius
        if distance < mindistance:
            return True
        return False

    def move(self):
        # impose speed limit...
        if self.velocity.x > SPEED_LIMIT:
            self.velocity.x = SPEED_LIMIT / 2
        elif self.velocity.x < -SPEED_LIMIT:
            self.velocity.x = -SPEED_LIMIT / 2

        if self.velocity.y > SPEED_LIMIT:
            self.velocity.y = SPEED_LIMIT / 2
        elif self.velocity.y < -SPEED_LIMIT:
            self.velocity.y = -SPEED_LIMIT / 2

        if self.velocity.z > SPEED_LIMIT:
            self.velocity.z = SPEED_LIMIT / 2
        elif self.velocity.z < -SPEED_LIMIT:
            self.velocity.z = -SPEED_LIMIT / 2

        self.pos += self.velocity
        self.updateEdges()

    def updateEdges(self):
        for edge in self.edges:
            edge.updatePosition()

    def applyForces(self, coeff=1):
        raise(Exception("IMPLEMENT ME: applyForces"))

    def __repr__(self):
        return "Node: %x (%s)  pos=%s, vel=%s, force=%s, color=%s" % (self.ident, self.__class__.__name__, repr(self.pos), repr(self.velocity), repr(self.impulse), repr(self.color) )




class Edge(cylinder):
    def __init__(self, nodepairtup):
        n1,n2 = nodepairtup
        cylinder.__init__(self, pos=n1.pos, axis=(n2.pos-n1.pos), radius=.1)

        self.nodes = nodepairtup
        for node in nodepairtup:
            node.edges.append(self)

        #print >>sys.stderr,("===== NEW EDGE CREATION ===== %s %s" % (nodepairtup))
        self.updatePosition()

    def updatePosition(self):
        n1,n2 = self.nodes
        self.pos=n1.pos
        self.axis=(n2.pos-n1.pos)

    def __repr__(self):
        n1,n2 = self.nodes
        return "".join([ "edge:", hex(n1.ident),"<->", hex(n2.ident), "  ", repr(self.pos), "/", repr(self.axis) ])

class System:
    def __init__(self):
        self.floor = box (pos=(0,0,0), length=400, height=0.5, width=400, color=(.05,.001,.3))
        self.nodes = {}
        self.edges = {}
        self.ceiling = 7
        self.smallest = 0xfffffffffffffffff
        self.minimum_safe_distance = 1
        self.time_on = True
        self.thread = threading.Thread(target=self.run)
        self.thread.setDaemon(True)
        self.thread.start()

    def changeCeiling(self, delta=10):
        self.ceiling += delta
        for node in self.nodes.values():
            node.y += delta
        for edge in self.edges.values():
            edge.updatePosition()

    def go(self):
        self.time_on = True

    def stop(self):
        self.time_on = False

    def addNode(self, node):
        #self.ceiling = 7 * len(self.nodes) + 2
        self.nodes[node.ident] = node
        if node.ident < self.smallest:
            self.smallest = node.ident

        #self.adjust()

    def addEdge(self, node1, node2, cls=Edge):
        edge = cls((node1, node2))
        if self.edges.get(edge.nodes) != None:
            raise(Exception("FAIL!  this node exists: %s" % repr(edge)))

        print >>sys.stderr,("adding new edge: %s" % repr(edge))

        self.edges[edge.nodes] = edge
        #self.adjust()

    def adjust(self):
        #go = True
        #print >>sys.stderr,("adjusting system.")
        #pendulum = .99
        #while go:
        #adjustment = vector(0,0,0)
        adjustment = 0

        for node in self.nodes.values():
            #print >>sys.stderr, node
            diff = node.applyForces()  # pendulum)
            adjustment += abs(diff)

        #x,y,z = adjustment
        #value = x+y+z
        #value = adjustment
        #if abs(value) <= STABILITY_CONST:
        #    go = False
        #else:
        #    print >>sys.stderr,("adjusting system.  adjustment = %s" % repr(value))
        #    #sys.stdin.readline()
        #pendulum *= .99      # let's see how it does without this dampener

    def run(self):
        while True:
            while self.time_on:
                self.adjust()
                for node in self.nodes.values():
                    node.move()
                    if node.y == self.minimum_safe_distance:
                        self.changeCeiling(+10)
                time.sleep(.05)
            time.sleep(.1)



class CodeBlockNode(Node):
    def applyForces(self, coeff=1):
        s = self.system

        # next add up the forces acting on this node.  this is applied to velocity later.
        force = vector(0,0,0)

        # gravity pulling me toward base
        force.y -= GRAVITY_COEFF

        # just a little tug toward the center of the graph, to keep things from exploding
        force.x -= CENTER_PULL * self.x
        force.z -= CENTER_PULL * self.z

        # all my edges pulling me in a direction
        for e in self.edges:        # FIXME: this has to be broken...
            n1, n2 = e.nodes
            if (n1 == self):
                diff = n1 - n2
                n = n2
            else:
                diff = n2 - n1
                n = n1
            if not abs(diff):
                diff = vector(.0001, 0, .0001)

            f_dir = diff
            #f_str = EDGE_FORCE_COEFF * ( math.log(abs(diff), EDGE_FORCE_BASE) )
            f_str = EDGE_FORCE_COEFF #* (abs(diff)**2)
            # edge force should never be greater than distance...
            thisforce = ( f_dir * f_str )
            #thisforce = - ( EDGE_SPRING_COEFF * diff * (log( abs(diff) ) + 1) )
            #print >>sys.stderr,("  EDGE: %x to %x:  diff=%s, absdiff=%s, dir=%s, str=%s, force=%s" % (self.ident, n.ident, diff, abs(diff), f_dir, f_str, thisforce))
            force += (thisforce * coeff)


        # all the other nodes pushing me away
        for n in s.nodes.values():
            if n == self:
                continue
            diff = (self - n)
            if not abs(diff):
                diff = vector(.0001, 0, .0001)

            f_dir = diff
            f_str = NODE_FORCE_COEFF / abs(diff)
            thisforce = - ( f_dir * f_str )
            if abs(thisforce) < 0.01:
                thisforce = vector(0,0,0)
            #thisforce = NODE_FORCE_COEFF * diff / abs(diff)
            #print >>sys.stderr,("  NODE: %x to %x:  npos=%s, diff=%s  abs(diff)=%s thisforce=%s dir=%s, str=%s velocity=%s" % 
            #        (self.ident, n.ident, n.pos, diff, abs(diff), thisforce, f_dir, f_str, self.velocity))
            force += (thisforce * coeff)

            # bump-effect  (what happens if the other node comes into my space!?)
            if self.isCollision(n):
                m1 = self.radius * self.velocity
                m2 = n.radius * n.velocity
                # FIXME: finish momentum stuff here


        # bouncing off the floor
        if self.y <= s.minimum_safe_distance:
            self.y = s.minimum_safe_distance
            #self.velocity.y = -self.velocity.y
            # ok, enough bouncing....
            if self.velocity.y < 0:
                self.velocity.y = 0
                force.y = 0
        # ceiling
        if self.y > s.ceiling:
            self.y = s.ceiling
            #self.velocity.y = -self.velocity.y
            # ok, enough bouncing....
            if self.velocity.y > 0:
                self.velocity.y = 0
                force.y = 0

        #print >>sys.stderr,("  TOTAL FORCE: %s (%s)" % (force, abs(force)))
        
        # friction (yes, in free-fall in the air...  sue me
        force += FRICTION_COEFF *  -( self.velocity )
        
        
        # update my velocity based on force
        self.velocity += VELOCITY_COEFF * force

        # now, the one glued to the ceiling
        if s.smallest == self.ident:
            self.pos = vector(0, s.ceiling, 0)
            self.velocity = vector(0,0,0)
            force.y = 0         # FIXME: how will this work???  should we spell this out?

        return force

        


def test(system):
    last = None
    for x in xrange(0x8048000, 0x8058000, 0x1000):
        n = CodeBlockNode(system, x)
        system.addNode(n)
        if last != None:
            system.addEdge(n, last)

        last = n
        #
#        if ((x>>8) % 4 == 3):
#            nn = system.nodes.values()[-3]
#            if n != nn:
#                system.addEdge(n, nn)

if __name__ == "__main__":
    s=System()
    test(s)



"""
 'CENTER_PULL': 0.050000000000000003,
 'EDGE_FORCE_COEFF': 0.0030000000000000001,
 'FRICTION_COEFF': 0.10000000000000001,
 'GRAVITY_COEFF': 0.049000000000000002,
 'NODE_FORCE_COEFF': 0.69999999999999996,
 'SPEED_LIMIT': 10,
 'STABILITY_CONST': 2,
 'VELOCITY_COEFF': 0.050000000000000003,
"""
"""
 'CENTER_PULL': 0.050000000000000003,
 'EDGE_FORCE_COEFF': 0.0040000000000000001,
 'FRICTION_COEFF': 0.10000000000000001,
 'GRAVITY_COEFF': 0.00097999999999999997,
 'NODE_FORCE_COEFF': 0.050000000000000003,
 'SPEED_LIMIT': 10,
 'STABILITY_CONST': 2,
 'VELOCITY_COEFF': 0.10000000000000001,

"""



from vtrace import *
import sys


HEAPSTART = None
def getConnectedChain(me, minimum=10):
    global HEAPSTART
    heapstart = 0
    heaplen = 0
    heapend = 0
    if HEAPSTART:
        heapstart = HEAPSTART[0]
        heaplen = HEAPSTART[1]
    else:
        for m in me.getMaps():
            if '[heap]' in m[3]:  
                heapstart = m[0]
                heaplen = m[1]
    heapptr = heapstart
    heapend = heapstart + heaplen
    finished = 0
    chunks = None
    #print("HEAPSTART: %x\t\t HEAPEND: %x\t\t HEAPLEN: %x"%(heapstart,heapend, heaplen))
    while heapptr < heapend and finished < minimum:
        heapptr += 4
        chunks = []
        finished = recurseConnectedChain(me, heapptr, heapend, chunks)
        #if finished > 1: print("Finished Tracing Path:  HEAP: %x\t\tLevels: %d"%(heapptr,finished))
        # HEAP address from maps should be aligned correctly
    if not HEAPSTART:
        HEAPSTART = (chunks[0],heaplen+heapstart-chunks[0])
    return chunks
    #HEAPCHAIN = (heapptr, finished, chunks)
    #return (heapptr, finished, chunks)

def recurseConnectedChain(me, address, end, chunks, finished=0):
    chunk = me.readMemoryFormat(address, "L")[0]  & 0xfffffffe
    chunks.append(address)
    if (chunk > 0 and not (chunk & 2) and chunk < end - address):
        #print ("Recursing %x, chunk = %x, nextaddress: %x, level %d"%(address, chunk, address+chunk, finished))
        try:
            #print " Calling Recurse again..."
            finished += (recurseConnectedChain(me, address+chunk, end, chunks))
        except Exception,e:
            print "ERROR: %s "%e
            return 1
    return finished +1

#findConnectedChain(me)

def findNextHeap(me, address):
    chain = getConnectedChain(me)
    for x in xrange(1,len(chain)):
        if chain[x] > address and chain[x-1] <= address:
            return chain[x]




#!/usr/bin/python -i

from atlasutils.vtraceutils import  *
from atlasutils.smartprint import  *
import os,sys,time
try:
    from pefile import *
except:
    from Elf import *


class LibraryNotifier(vtrace.Notifier):
    """
    A small example notifier which prints
    out libraries as they are loaded.
    """
    def notify(self, event, trace):
        if event == vtrace.NOTIFY_LOAD_LIBRARY:
            print("-----Library Loaded:",trace.getMeta("LatestLibrary"))


class tracebin:

    def __init__(self, name, trace=None, verbose = False):
        self.binexe = name.split(os.sep)[-1]
        self.verbose = verbose
        self.locsec = None
        self.bin = None
        self.startaddress = None
        self.eip = 0
        self.lastcall = (0,0)   # last target and last source address
        if os.name == 'posix':
            self.bin = Elf(argv[1])
            self.startaddress = self.bin.e_entry
        elif os.name == 'nt':
            self.bin = PE(argv[1])
            self.startaddress = self.bin.OPTIONAL_HEADER.AddressOfEntryPoint + self.bin.OPTIONAL_HEADER.ImageBase
        
        self.me = getTrace()
        self.me.registerNotifier(NOTIFY_EXIT, VerboseNotifier())
        self.me.registerNotifier(NOTIFY_BREAK, VerboseNotifier())
        self.me.registerNotifier(NOTIFY_LOAD_LIBRARY, LibraryNotifier())
        self.me.addBreakpoint(Breakpoint(self.startaddress))      # This should get us past the loader code into the actual start of the binary
        self.me.metadata['RUN'] = True
        self.me.execute(argv[1])
        
        #self.startaddress += 2           # skip prolog
        
            
    def isLocal(self, address):
        if not self.locsec:
            if os.name[0] == 'p':
                self.genLocalLinux()
            elif os.name[0] == 'n':
                self.genLocalWin()
            else:
                self.genLocalDefault()
                
        for x in self.locsec:
            if address >= x[0] and address <= x[1]:
                return True
        return False
    
    def genLocalLinux(self):
        self.locsec = []
        for sec in self.bin.getSections():
            if (sec.sh_flags & 4 and sec.name[:4] != '.plt' and sec.name[:5] != '.init'):
                start = sec.sh_addr
                end   = sec.sh_addr+sec.sh_size
                print("    appending 'local' section...  %s (%x - %x)"%(sec.name, start, end))
                self.locsec.append((start, end))
        return True
            
    def genLocalWin(self):
        self.locsec = []
        for sec in self.bin.sections:
            if (sec.Name[:5].lower() == '.text' or sec.Characteristics & 0x20000000 ):
                start = sec.VirtualAddress + self.bin.OPTIONAL_HEADER.ImageBase
                end   = sec.VirtualAddress + self.bin.OPTIONAL_HEADER.ImageBase + sec.SizeOfRawData
                print("    appending 'local' section...  %s (%x-%x)"%(sec.Name, start, end))
                self.locsec.append((start, end))
        return True
    def genLocalDefault(self):
        #locsec = []
        mytextstart = 0xffffffff
        mytextend = 0
        for x in self.me.getMemoryMaps():
            if binexe in x[3]:
                print("Tracing:  %s compare %x to %s at %x"%(binexe,mytextstart,x[3], x[0]))
                if mytextstart > x[0]:
                    mytextstart = x[0]
                if mytextend < (x[0]+x[1]):
                    mytextend = (x[0]+x[1])
        self.locsec = [(mytextstart,mytextend)]
        
    def getOperandValue(self, operand):
        dereference = False
        add = ""
        numaddr = 0
        source = operand
        if isinstance(source, SIB):
            if source.index:
                numaddr += (source.index * source.scale)
            if source.base:
                source = source.base
            #print("DEBUG-sib: %x"%numaddr, file=outfile)
        if isinstance(source, Expression):
            if source.disp:
                numaddr += source.disp.value
            if source.base:
                source = source.base
            dereference = True
            #print("DEBUG-expr: %x"%numaddr, file=outfile)
        if isinstance(source, SIB):
            if source.index:
                if isinstance(source.index, Register):
                    numaddr += (self.getRegisterValue(source.index))
                else:
                    print(source.index)
                    numaddr += (source.index * source.scale)
            if source.base:
                source = source.base
            #print("DEBUG-sib: %x"%numaddr, file=outfile)
        if isinstance(source, Register):
            numaddr += (self.getRegisterValue(source))
            #print("DEBUG-reg: %x"%numaddr, file=outfile)
        elif isinstance(source, Address):
            numaddr += source.value
            if source.relative:
                numaddr += self.eip + self.op.off
            #print("DEBUG-addr: %x"%numaddr, file=outfile)
        if dereference:
            try:
                numaddr = struct.unpack("L",me.readMemory(numaddr, 4))[0]
                #print("DEBUG-deref: %x"%numaddr, file=outfile)
            except:
                #print("DEBUG-deref-except: %x"%numaddr, file=outfile)
                #print("DEBUG-deref-except: %x"%numaddr, file=stderr)
                pass
                
        return numaddr
        
    def getRegisterValue(self, source):
            reg = source.name
            mask = 0xffffffff       # 32 bit only
            if reg[0] != 'e' and reg[1] in ['l','h','x']:       # TOTALLY libdisassemble-centric, as it assumes lowercase letters
                if reg[1] == 'x':
                    mask = 0xffff
                elif reg[1] == 'l':
                    mask = 0xff
                elif reg[1] == 'h':
                    mask = 0xff00
                reg = 'e%cx'%reg[0]                             # vtrace doesn't like al/ah and possibly ax
            return (self.me.getRegisterByName(reg) & mask)
            #print(("DEBUG-reg: %x"%numaddr), file=outfile)

    def localstepi(self, outfile = stdout):
        me = self.me
        origThread = me.getMeta('ThreadId')
        threads = None
        while threads == None:
            try:
                threads = me.getThreads()
            except:
                print(("Error: Thread list changed in middle of getThreads()"), file=stderr)
        for t in threads:
            values = ""
            me.selectThread(t)
            me.stepi()
            self.eip = me.getProgramCounter()
            #print(("debug: %x"%self.eip), file=outfile)
            try:
              self.op = Opcode(me.readMemory(self.eip, 18))
              if self.isLocal(self.eip):
                # LOCAL TO THIS BINARY's .TEXT SECTION
                values = "; "
                if self.op.dest:
                    if not isinstance(self.op.dest, Address):
                        num = self.getOperandValue(self.op.dest)
                    else:
                        num = self.op.dest.value
                        if self.op.dest.relative:
                            num += self.eip
                    temp = "\t%x"%num
                    if me.getMemoryMap(num) != None:
                        try:
                            temp += "=('%s'...)"%(hexText(me.readMemory(num, 10)))
                        except:
                            print("ERROR READING MEMORY AT %x:"%num)
                    values += " %s"%(temp)
                if self.op.source:
                    if not isinstance(self.op.source, Address):
                        num = self.getOperandValue(self.op.source)
                    else:
                        num = self.op.source.value
                        if self.op.source.relative:
                            num += self.eip
                    temp = "\t%x"%num
                    if me.getMemoryMap(num) != None:
                        try:
                            temp += "=('%s'...)"%(hexText(me.readMemory(num, 10)))
                        except:
                            print("ERROR READING MEMORY AT %x:"%num)
                    values += " %s"%(temp)
              print("thread %8d: %x   (%s)\t\t %s \t%s"%(t,self.eip,me.getSymByAddr(self.eip), self.op.printOpcode(0, self.eip), values), file=outfile)
              if self.op.opcode[:2] == "ca" or (self.op.opcode[0] == 'j' and abs(self.lastcall[0] - self.eip) < 4):
                dereference = False
                add = ""
                numaddr = 0
                source = self.op.source
                if isinstance(source, SIB):
                    numaddr += BREAKIT + source.index
                    source = source.index
                    print(("DEBUG-sib: %x"%numaddr), file=outfile)
                if isinstance(source, Expression):
                    if source.disp:
                        numaddr += source.disp.value
                    if source.base:
                        source = source.base
                    dereference = True
                    print(("DEBUG-expr: %x"%numaddr), file=outfile)
                if isinstance(source, Register):
                    numaddr += self.me.getRegisterByName(source.name)
                    print(("DEBUG-reg: %x"%numaddr), file=outfile)
                elif isinstance(source, Address):
                    numaddr += source.value
                    if source.relative:
                        numaddr += self.eip + self.op.off
                    print(("DEBUG-addr: %x"%numaddr), file=outfile)
                
                if dereference:
                    try:
                        numaddr = struct.unpack("L",me.readMemory(numaddr, 4))[0]
                        print(("DEBUG-deref: %x"%numaddr), file=outfile)
                    except:
                        print(("DEBUG-deref-except: %x"%numaddr), file=outfile)
                    
                target = me.getSymByAddr(numaddr)
                print( ("\t\t %x(%s) %s TO %s -- %x(%s)  \t\t\t (%s)"%(self.eip, self.isLocal(self.eip),self.op.opcode, self.op.source.printOpcode(0, self.eip), numaddr, self.isLocal(numaddr), target)), file=outfile)
                try:    ### LINUX ONLY... fugly
                    if lookupPLT(numaddr) == '__libc_start_main':
                        main = struct.unpack("L", me.readMemory(me.getRegisterByName('esp'), 4))[0]
                        print(("\t\t Setting main() Breakpoint at %x"%(main)), file=outfile)
                        me.addBreakpoint(Breakpoint(main))
                except:
                    pass
                if not self.isLocal(numaddr) and self.isLocal(self.eip):
                    if self.op.opcode[0] == 'j':
                        baddr = self.lastcall[1]
                    else:
                        baddr = self.eip+self.op.off
                    brk = Breakpoint(baddr)
                    print(("\t\t Going Remote...  Setting Breakpoint at %x"%(baddr)), file=outfile)
                    try:
                        me.addBreakpoint(brk)
                    except:
                        pass
                    me.run()
                    print(repr(me))
                    me.removeBreakpoint(brk)
                else:
                    self.lastcall = (numaddr, self.eip + self.op.off)
                #except:
                    #print(("ERROR READING MEMORY LOCATION: %x"%eip), file=stderr)
            except:
                x,y,z = sys.exc_info()
                sys.excepthook(x,y,z)
        me.selectThread(origThread)
    
    
    
    
    
    
    
    def go(self, init = True):
        me = self.me
        #if init:
        print("Placing initial breakpoint at Entry Point: %x"%self.startaddress)
            #me.addBreakpoint(Breakpoint(self.startaddress))      # This should get us past the loader code into the actual start of the binary
            #while self.eip != self.startaddress:
            #    me.run()
            #    self.eip = me.getProgramCounter()
            #    print("Initial Stop at %x"%self.eip)
            #    time.sleep(1)
        
        if os.name =='nt':          # TAKE CARE OF ANTIDEBUGGING UNDER WINDOWS
            """
                >>> me.getSymByName('IsDebuggerPresent', 'kernel32')
                >>> idp=me.getSymByName('IsDebuggerPresent', 'kernel32')
                >>> while idp == None:
                    idp=me.getSymByName('IsDebuggerPresent', 'kernel32')
                    me.stepi()
                
                    
                >>> me.getProgramCounter()
                2089868385
                >>> "%x"%me.getProgramCounter()
                '7c90dc61'
                >>> 
            """
            idp = None
            print("Tracing until Kernel32 is loaded...")
            while idp == None:
                me.stepi()
                idp=me.getSymByName('IsDebuggerPresent', 'kernel32')
            print("Overwriting 'IsDebuggerPresent' to hide results")
            idp=me.getSymByName('isdebuggerpresent','kernel32').value
            me.writeMemory(idp, '\x33\xc0\xc3')     # %eax = 0, return
            
        while self.me.getMeta('RUN'): 
            try:
                self.localstepi()
            except KeyboardInterrupt:
                self.me.setMeta('RUN',False)
if __name__ == "__main__":                
  while (argv[1][0] == '-'):
    arg = argv.pop(1)
    if arg[1] == 'V':
        self.verbose = True
    else:
        print(("ERROR: Unknown Parameter: %s"%arg), file=stderr)

  name = argv[1]
  tracebin(argv).go()

